import cn.estsh.i3plus.platform.common.tool.TimeTool
import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper
import cn.estsh.i3plus.pojo.mes.repository.MesWorkOrderToSapRepository
import lombok.Getter
import lombok.Setter
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate

import javax.annotation.Resource
import java.sql.Connection

/**
 * QMS同步MES_CCSC任务检测结果
 * @Author : gsz
 * @CreateDate 2024/5/23 9:58
 */
class sapToMesCcscTaskResult {

    public static final Logger LOGGER = LoggerFactory.getLogger(sapToMesCcscTaskResult.class)

    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;
    @Autowired
    private MesWorkOrderToSapRepository mesWorkOrderToSapRDao;

    @Resource(name = "yfasDataSource")
    private DynamicDataSourceProxy yfasDataSource;

    @Resource(name = "mesDataSource")
    private DynamicDataSourceProxy mesDataSource;

    @Getter
    @Setter
    private Connection mesConn;

    def filterData(MesInterfaceDataMapper mapper, List<Map<String, Object>> srcData) throws Exception {
        if (srcData == null || srcData.size() == 0) {
            return srcData
        }
        Map<String, Map<String, Object>> destMapData = new HashMap<>();
        LOGGER.info("-------- filterData Start QMS同步MES_CCSC任务检测结果 ---------------")

        List<Map<String, Object>> destData = new ArrayList<>();
        List<String> insertSqlList = new ArrayList<>();
        for (Map<String, Object> rowMap : srcData) {
            String organizeCode = String.valueOf(rowMap.get("WERKS"));
            String taskNo = rowMap.get("task_no").toString();
            String taskResult = rowMap.get("task_result").toString(); //1:合格 2:不合格

            Map<String, Object> sqlParams = new HashMap<>(6)
            sqlParams.put("organizeCode", organizeCode);
            sqlParams.put("task_no", taskNo);
            StringBuffer sql = new StringBuffer("select * from  mes_ccsc_task where " +
                    " organize_code=:organizeCode and task_no=:task_no ; ");
            List<Map<String, Object>> resultMap = this.queryDataTable(sql.toString(), sqlParams)

            insertSqlList.add("update mes_ccsc_task set  task_result = '"+taskResult+ "'  " +
                    "  where organize_code= '"+organizeCode+"' and task_no= '"+taskNo+"'; ");

            LOGGER.info("-------- filterData Start QMS同步MES_CCSC任务检测结果 60----------")

            if (resultMap.size() > 0) {
                sqlParams.put("work_order_no", resultMap.get(0).get("work_order_no"));
                String part_no = resultMap.get(0).get("part_no")
                sqlParams.put("part_no", part_no);
                StringBuffer sqlWork = new StringBuffer("select * from  mes_work_order where " +
                        " organize_code=:organizeCode and work_order_no=:work_order_no and part_no=:part_no ; ");
                List<Map<String, Object>> resultWorkMap = this.queryDataTable(sqlWork.toString(), sqlParams)

                if (resultWorkMap.size() > 0) {
                    String work_center_code = resultWorkMap.get(0).get("work_center_code")
                    String work_cell_code = resultWorkMap.get(0).get("work_cell_code")

                    LOGGER.info("-------- filterData Start QMS同步MES_CCSC任务检测结果 70{}",work_center_code)
                    String logUpdateSql="update mes_comunication_log set  is_deleted = 1 " +
                            " where organize_code= '"+organizeCode+"' and work_center_code= '"+work_center_code+"' " +
                            " and work_cell_code= '"+work_cell_code+"' and parameter_name= '"+part_no+"'; "
                    insertSqlList.add(logUpdateSql);

                    if (taskResult.equals("2")) { //1:合格 2:不合格
                        String insertSql = String.format(" INSERT INTO mes_comunication_log " +
                                "(`parameter_name`,`parameter_desc`,`value`,`status`,`work_cell_code`,`work_center_code`, " +
                                "`id`,`organize_code`,`create_date_time`,`create_user` )" +
                                "VALUES('%s', '%s','%s','%s','%s','%s','%s','%s','%s','%s') ; "
                                , part_no, part_no, part_no, 1, work_cell_code, work_center_code,
                                snowflakeIdMaker.nextId(), organizeCode, TimeTool.getNowTime(true), "CCSC_SAP");
                        insertSqlList.add(insertSql);
                        //更新所有条码为可疑品
                        String sqlWorkSn =  "update mes_produce_sn set qc_status=30 " +
                                " where organize_code= '"+organizeCode+"' and work_center_code= '"+work_center_code+"' " +
                                " and work_cell_code= '"+work_cell_code+"' "+
                                " and part_no= '"+part_no+"'; ";
                        insertSqlList.add(sqlWorkSn);
                    }

                }
            }
            destData.add(rowMap);

        }
        //更新同步标识
        this.mesConn = mesDataSource.getWriteConnectionWithoutPool();
        mesDataSource.executeAsBatch(insertSqlList, mesConn)
        mesDataSource.closeConnectionWithoutPoll(this.mesConn)

        //更新同步标识
//        this.mesConn = yfasDataSource.getWriteConnectionWithoutPool();
//        yfasDataSource.executeAsBatch(updateSqlList,mesConn)
//        yfasDataSource.closeConnectionWithoutPoll(this.mesConn)
        return destData;
    }

    private List<Map<String, Object>> queryDataTable(String sql, Map<String, Object> params) {
        NamedParameterJdbcTemplate namedJdbcTemplate = new NamedParameterJdbcTemplate(mesDataSource.getDataSource())

        List<Map<String, Object>> dataMap = namedJdbcTemplate.queryForList(sql, params)

        return dataMap;
    }
}