package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderToSapService;
import cn.estsh.i3plus.ext.mes.apiservice.config.AuthUtilExt;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel.MesExcelTool;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.i3plus.pojo.model.wms.WmsProductModel;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

@Api(description = "工单主页面")
@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/mesWorkOrder")
public class MesWorkOrderController extends BaseMesController<MesWorkOrder> {

    @Autowired
    private IMesWorkOrderService mesWorkOrderService;

    @Autowired
    private IMesWorkOrderToSapService mesWorkOrderToSapService;

    @PostMapping(value = "/import-mes-work-order")
    @ApiOperation(value = "导入数据")
    public ResultBean importMesWorkOrderExcel(@RequestParam("file") MultipartFile file, @RequestParam("status") Integer type) {
        try {
            MesExcelTool excelTool = new MesExcelTool(entityManager, RedisCacheTool.getImppRedis());
            List<MesWorkOrder> beanList = excelTool.importData(file.getOriginalFilename(), file.getInputStream(), MesWorkOrder.class);

            // 校验导入数据
            validateImport(beanList);
            String userName = AuthUtil.getSessionUser().getUserName();
            String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
            //非排序工单导入前删除
            if (MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue() == type) {
                mesWorkOrderService.deleteUnSortOrder(userName,organizeCode);
            }
            // 导入数据初始化
            for (MesWorkOrder bean : beanList) {
                bean.setWorkOrderType(type);
                ConvertBean.serviceModelInitialize(bean, userName);
                bean.setOrganizeCode(organizeCode);
            }
            baseService.insertBatch(beanList);

            // 导入后
            afterImport(beanList);
            return ResultBean.success("导入成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/doProductReport")
    @ApiOperation(value = "生产报工")
    public ResultBean doProductReport(@RequestBody MesWorkOrder mesWorkOrder) {
        try {
            //新增初始化
            mesWorkOrder.setOrganizeCode(AuthUtil.getOrganizeCode());
            ConvertBean.serviceModelInitialize(mesWorkOrder, AuthUtil.getSessionUser().getUserName());
            mesWorkOrderService.doProductReport(mesWorkOrder);
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException e) {
            return ResultBean.fail(e).build();
        } catch (Exception e) {
            return ResultBean.fail(e);
        }
    }

    @PostMapping(value = "/doProductReportExt")
    @ApiOperation(value = "生产报工(武汉)")
    public ResultBean doProductReportExt(@RequestBody MesWorkOrder mesWorkOrder) {
        try {
            ValidatorBean.checkNotNull(mesWorkOrder.getId(),"工单id不能为空");
            ValidatorBean.checkNotNull(mesWorkOrder.getNum(),"报工数量不能为空");
            ValidatorBean.checkNotZero(mesWorkOrder.getNum(),"报工数量不能为0");
            mesWorkOrder.setOrganizeCode(AuthUtilExt.getOrganizeCode());
            ConvertBean.serviceModelUpdate(mesWorkOrder, AuthUtilExt.getUserName());
            //新增初始化
            mesWorkOrderService.doProductReportExt(mesWorkOrder);
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException e) {
            return ResultBean.fail(e).build();
        } catch (Exception e) {
            return ResultBean.fail(e);
        }
    }


    @GetMapping(value = "/sendToSap")
    @ApiOperation(value = "发送给sap")
    public ResultBean sendToSap(String organizeCode) {
        try {
            //查询汇总工单数量
            List<MesWorkOrder> mesWorkOrderList = mesWorkOrderService.sumMesWorkOrderQtyToSap(organizeCode);
            if (!CollectionUtils.isEmpty(mesWorkOrderList)){
                mesWorkOrderToSapService.doMesWorkOrder(organizeCode,mesWorkOrderList);
            }
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException e) {
            return ResultBean.fail(e).build();
        } catch (Exception e) {
            return ResultBean.fail(e);
        }
    }

    @PostMapping(value = "/report-work-reversal")
    @ApiOperation(value = "反向冲销")
    public ResultBean doReportWorkReversal(@RequestBody WmsProductModel model) {
        try {
            ValidatorBean.checkNotNull(model.getOrganizeCode(),"工厂代码不能为空");
            ValidatorBean.checkNotNull(model.getQty(),"数量不能为空");
            ValidatorBean.checkNotNull(model.getSn(),"条码不能为空");
            ValidatorBean.checkNotNull(model.getPartNo(),"零件号不能为空");

            mesWorkOrderService.doReportWorkReversal(model);
            return ResultBean.success("反向冲销成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


}
