package cn.estsh.i3plus.ext.mes.apiservice.controller.dbinterface;

import cn.estsh.i3plus.ext.mes.apiservice.dbinterface.MesSAPDbAdapter;
import cn.estsh.i3plus.icloud.softswitch.sdk.IBsSuitServiceCloud;
import cn.estsh.i3plus.mes.apiservice.serviceimpl.engine.script.EngineScriptManager;
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * @Description : SAP接口同步控制器
 * @Reference :
 * @Author : gsz
 * @CreateDate : 2024-05-10 12:08
 * @Modify:
 **/
@RestController
@RequestMapping("/impp/white/mes-sap-if/")
@Api(description = "SAP接口同步控制器")
public class WmsSAPDbInterfaceController {
    private static final Logger LOGGER = LoggerFactory.getLogger(WmsSAPDbInterfaceController.class);

    @Autowired
    IBsSuitServiceCloud bsSuitServiceCloud;

    // 注入脚本引擎对象
    @Autowired
    private EngineScriptManager scriptManager;

    @GetMapping(value = "sync-data/do")
    @ApiOperation(value = "执行接口数据同步，支持的参数：SAP2MES, MES2SAP ")
    public ResultBean doSyncSAPData(String syncParam) {
        try {
            // 判断作业参数是否支持
            if (MesInterfaceEnumUtil.DIRECTION_TYPE.nameOf(syncParam) == -1) {
                throw new IllegalArgumentException("不支持的同步参数：" + syncParam);
            }
            LOGGER.info("WmsSAPDbInterfaceController.doSyncSAPData start.....");
            MesSAPDbAdapter wmsSAPDbAdapter = new MesSAPDbAdapter();
            // new 的对象需要手工注入 bean
            if (SpringContextsUtil.getApplicationContext() != null) {
                SpringContextsUtil.getApplicationContext().
                        getAutowireCapableBeanFactory().autowireBean(wmsSAPDbAdapter);
            }
            wmsSAPDbAdapter.syncData(syncParam,null);
            LOGGER.info("WmsSAPDbInterfaceController.doSyncSAPData end.....");

            return new ResultBean(true).setMsg("SAP-->WMS接口数据同步执行成功");
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e).setResultObject(e);
        }
    }

    @GetMapping(value = "sync-data-id/do")
    @ApiOperation(value = "按ID单个执行接口数据同步，支持的参数：SAP2MES, MES2SAP ")
    public ResultBean doSyncSAPData(String syncParam, String id) {
        try {
            // 判断作业参数是否支持
            if (MesInterfaceEnumUtil.DIRECTION_TYPE.nameOf(syncParam) == -1) {
                throw new IllegalArgumentException("不支持的同步参数：" + syncParam);
            }
            LOGGER.info("WmsSAPDbInterfaceController.doSyncSAPData start.....");
            // 开始同步数据
            // 一定要 new 对象，不然会有并发问题
            MesSAPDbAdapter wmsSAPDbAdapter = new MesSAPDbAdapter();
            // new 的对象需要手工注入 bean
            if (SpringContextsUtil.getApplicationContext() != null) {
                SpringContextsUtil.getApplicationContext().
                        getAutowireCapableBeanFactory().autowireBean(wmsSAPDbAdapter);
            }
            wmsSAPDbAdapter.syncData(syncParam, id);
            LOGGER.info("WmsSAPDbInterfaceController.doSyncSAPData end.....");

            return new ResultBean(true).setMsg("SAP-->WMS接口数据同步执行成功");
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e).setResultObject(e);
        }
    }


}
