package cn.estsh.i3plus.ext.mes.apiservice.cus_suit;

import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy;
import lombok.Getter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.concurrent.TimeUnit;

/**
 * @Description : 客户定制化配置，对应配置文件
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2019-06-11 14:26
 * @Modify:
 **/
@Configuration
//@PropertySource(value="classpath:/cus_config/cus-datasource.properties",encoding = "UTF-8")
public class CusProperties {

    public static final Logger LOGGER = LoggerFactory.getLogger(CommonConstWords.SYSTEM_LOG);

    /***********************
     * 安道拓数据源配置
     */
    @Value("${impp.yfas.datasource.isopen}")
    @Getter
    private boolean yfasIsOpen;

    @Value("${impp.yfas.datasource.driver-class-name}")
    @Getter
    private String yfasDriverClassName;

    @Value("${impp.yfas.datasource.alias}")
    @Getter
    private String yfasDsAlias;

    @Value("${impp.yfas.datasource.jdbc-url}")
    @Getter
    private String yfasDsJdbcUrl;

    @Value("${impp.yfas.datasource.username}")
    @Getter
    private String yfasDsUsername;

    @Value("${impp.yfas.datasource.password}")
    @Getter
    private String yfasDsPassword;

    @ConditionalOnExpression("'${impp.yfas.datasource.isopen:false}' == 'true'")
    @Bean(name = "yfasDataSource")
    public DynamicDataSourceProxy initYfasCustomerSet(){
        LOGGER.info("【初始化客户定制化...】");

        /**
         * 注册客户定制数据源
         */
        LOGGER.info("【客户定制化：安道拓数据源加载...】");
        //安道拓定制业务
        DynamicDataSourceProxy yfasDataSource = DynamicDataSourceProxy.initDataSourceFactory(this.getYfasDriverClassName(),
                this.getYfasDsJdbcUrl(),this.getYfasDsUsername(),this.getYfasDsPassword(),TimeUnit.MINUTES.toMillis(30));

        //手动注册DataSource到spring中
        //SpringContextsUtil.registBean(this.getYfasDsAlias(),yfasDataSource);

        return yfasDataSource;
    }
}
