package cn.estsh.i3plus.ext.mes.apiservice.daoimpl;

import cn.estsh.i3plus.ext.mes.apiservice.dao.IEquipmentExtDao;
import cn.estsh.i3plus.mes.apiservice.dao.IEquipmentDao;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipment;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCenter;
import cn.estsh.i3plus.pojo.mes.model.MesEquipmentModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-12
 * @Modify:
 **/
@Service
public class EquipmentExtDao implements IEquipmentExtDao {
    public static final Logger LOGGER = LoggerFactory.getLogger(EquipmentExtDao.class);

    @Autowired
    private EntityManager entityManager;

    /**
     * 获取设备分页数据
     * @param mesEquipment
     * @param pager
     * @return
     */
    @Override
    public List<MesEquipmentModel> queryMesEquipmentByPager(MesEquipment mesEquipment, Pager pager) {
        StringBuffer hql = new StringBuffer();
        hql.append(" select distinct new " + MesEquipmentModel.class.getName() + "(equ.id, equ.equipmentCode, equ.equipmentName, " +
                " mwe.workCenterCode, mwc.areaCode, equ.equipmentType, " +
                " equ.equipmentModel, equ.equipmentSpec, equ.equipmentMaker, equ.releaseDate, equ.receiveDate, equ.enableDate, equ.connectType, " +
                " equ.organizeCode, equ.isValid, equ.isDeleted, equ.createUser, equ.createDatetime, equ.modifyUser, equ.modifyDatetime, equ.checkModel, equ.fixedAssetsNo) ");
        hql.append(" from MesEquipment as equ ");
        hql.append(" left join MesWcEquipment as mwe on (equ.equipmentCode = mwe.equipmentCode and mwe.organizeCode = :organizeCode and " +
                " mwe.isValid = :isValid1 and mwe.isDeleted = :isDeleted) ");
        hql.append(" left join MesWorkCenter as mwc on (mwe.workCenterCode = mwc.workCenterCode and mwc.organizeCode = :organizeCode and " +
                " mwc.isValid = :isValid1 and mwc.isDeleted = :isDeleted) ");
        hql.append(" where equ.organizeCode = :organizeCode ");
        hql.append(" and equ.isValid = :isValid ");
        hql.append(" and equ.isDeleted = :isDeleted ");
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentCode())) {
            hql.append(" and equ.equipmentCode like :equipmentCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentName())) {
            hql.append(" and equ.equipmentName like :equipmentName ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            hql.append(" and mwe.workCenterCode = :workCenterCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            hql.append(" and mwe.workCellCode = :workCellCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getAreaCode())) {
            hql.append(" and mwc.areaCode = :areaCode ");
        }
        hql.append(" group by equ.id, equ.equipmentCode, equ.equipmentName, " +
                " mwe.workCenterCode, mwc.areaCode, equ.equipmentType, equ.equipmentCategory, equ.memo, " +
                " equ.equipmentModel, equ.equipmentSpec, equ.equipmentMaker, equ.releaseDate, equ.receiveDate, equ.enableDate, equ.connectType, " +
                " equ.organizeCode, equ.isValid, equ.isDeleted, equ.createUser, equ.createDatetime, equ.modifyUser, equ.modifyDatetime, equ.checkModel, equ.fixedAssetsNo ");
        hql.append(" order by equ.modifyDatetime desc ");
        Query query = entityManager.createQuery(hql.toString(), MesEquipmentModel.class);
        query.setParameter(MesConstWords.ORGANIZE_CODE, mesEquipment.getOrganizeCode());
        query.setParameter(MesConstWords.IS_VALID, mesEquipment.getIsValid());
        query.setParameter("isValid1", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentCode())) {
            query.setParameter("equipmentCode", "%" + mesEquipment.getEquipmentCode() + "%");
        }
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentName())) {
            query.setParameter("equipmentName", "%" + mesEquipment.getEquipmentName() + "%");
        }

        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            query.setParameter("workCenterCode", mesEquipment.getWorkCenterCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            query.setParameter("workCellCode", mesEquipment.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getAreaCode())) {
            query.setParameter("areaCode", mesEquipment.getAreaCode());
        }
        query.setMaxResults(pager.getPageSize());  //设置获取的数量
        if (pager.getCurrentPage() == 0) {
            pager.setCurrentPage(1);
        }
        query.setFirstResult(pager.getCurrentPage() * pager.getPageSize() - pager.getPageSize());
        return query.getResultList();
    }

    /**
     * 获取设备分页数据数量
     * @param mesEquipment
     * @return
     */
    @Override
    public int queryMesEquipmentByPagerCount(MesEquipment mesEquipment) {
        StringBuffer hql = new StringBuffer();
        hql.append(" select count(distinct equ.id) from MesEquipment as equ ");
        hql.append(" left join MesWcEquipment as mwe on (equ.equipmentCode = mwe.equipmentCode and mwe.organizeCode = :organizeCode and " +
                " mwe.isValid = :isValid1 and mwe.isDeleted = :isDeleted) ");
        hql.append(" left join MesWorkCenter as mwc on (mwe.workCenterCode = mwc.workCenterCode and mwc.organizeCode = :organizeCode and " +
                " mwc.isValid = :isValid1 and mwc.isDeleted = :isDeleted) ");
        hql.append(" where equ.organizeCode = :organizeCode ");
        hql.append(" and equ.isValid = :isValid ");
        hql.append(" and equ.isDeleted = :isDeleted ");
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentCode())) {
            hql.append(" and equ.equipmentCode like :equipmentCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentName())) {
            hql.append(" and equ.equipmentName like :equipmentName ");
        }

        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            hql.append(" and mwe.workCenterCode = :workCenterCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            hql.append(" and mwe.workCellCode = :workCellCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getAreaCode())) {
            hql.append(" and mwc.areaCode = :areaCode ");
        }
        Query query = entityManager.createQuery(hql.toString(), Long.class);
        query.setParameter(MesConstWords.ORGANIZE_CODE, mesEquipment.getOrganizeCode());
        query.setParameter(MesConstWords.IS_VALID, mesEquipment.getIsValid());
        query.setParameter("isValid1", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentCode())) {
            query.setParameter("equipmentCode", "%" + mesEquipment.getEquipmentCode() + "%");
        }
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentName())) {
            query.setParameter("equipmentName", "%" + mesEquipment.getEquipmentName() + "%");
        }

        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            query.setParameter("workCenterCode", mesEquipment.getWorkCenterCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            query.setParameter("workCellCode", mesEquipment.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getAreaCode())) {
            query.setParameter("areaCode", mesEquipment.getAreaCode());
        }
        return ((Long) query.getSingleResult()).intValue();
    }

    /**
     * 获取设备分页数据-组织模型
     * @param mesEquipment
     * @param pager
     * @return
     */
    @Override
    public List<MesEquipmentModel> queryMesEquipmentByPagerOrg(MesEquipment mesEquipment, Pager pager) {
        StringBuffer hql = new StringBuffer();
        hql.append(" select new " + MesEquipmentModel.class.getName() + "(mwe.id as wcId, equ.id, equ.equipmentCode, " +
                " equ.equipmentName,  mwe.workCenterCode, mwe.workCellCode, mwc.areaCode)");
        hql.append(" from MesEquipment as equ ");
        hql.append(" right join MesWcEquipment as mwe on (equ.equipmentCode = mwe.equipmentCode and mwe.organizeCode = :organizeCode and " +
                " mwe.isValid = :isValid and mwe.isDeleted = :isDeleted) ");
        hql.append(" left join MesWorkCenter as mwc on (mwe.workCenterCode = mwc.workCenterCode and mwc.organizeCode = :organizeCode and " +
                " mwc.isValid = :isValid1 and mwc.isDeleted = :isDeleted) ");
        hql.append(" where equ.organizeCode = :organizeCode ");
        hql.append(" and equ.isValid = :isValid1 ");
        hql.append(" and equ.isDeleted = :isDeleted ");
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentCode())) {
            hql.append(" and mwe.equipmentCode like :equipmentCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentName())) {
            hql.append(" and equ.equipmentName like :equipmentName ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            hql.append(" and mwe.workCenterCode = :workCenterCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            hql.append(" and mwe.workCellCode = :workCellCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getAreaCode())) {
            hql.append(" and mwc.areaCode = :areaCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getId())) {
            hql.append(" and mwe.id = :id ");
        }
        hql.append(" order by equ.modifyDatetime desc ");
        Query query = entityManager.createQuery(hql.toString(), MesEquipmentModel.class);
        query.setParameter(MesConstWords.ORGANIZE_CODE, mesEquipment.getOrganizeCode());
        query.setParameter(MesConstWords.IS_VALID, mesEquipment.getIsValid());
        query.setParameter("isValid1", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentCode())) {
            query.setParameter("equipmentCode", "%" + mesEquipment.getEquipmentCode() + "%");
        }
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentName())) {
            query.setParameter("equipmentName", "%" + mesEquipment.getEquipmentName() + "%");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            query.setParameter("workCenterCode", mesEquipment.getWorkCenterCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            query.setParameter("workCellCode", mesEquipment.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getAreaCode())) {
            query.setParameter("areaCode", mesEquipment.getAreaCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getId())) {
            query.setParameter("id", mesEquipment.getId());
        }
        query.setMaxResults(pager.getPageSize());  //设置获取的数量
        query.setFirstResult(pager.getCurrentPage() * pager.getPageSize() - pager.getPageSize());
        return query.getResultList();
    }

    /**
     * 获取设备分页数据数量-组织模型
     * @param mesEquipment
     * @return
     */
    @Override
    public int queryMesEquipmentByPagerOrgCount(MesEquipment mesEquipment) {
        StringBuffer hql = new StringBuffer();
        hql.append(" select count(equ.id) ");
        hql.append(" from MesEquipment as equ ");
        hql.append(" right join MesWcEquipment as mwe on (equ.equipmentCode = mwe.equipmentCode and mwe.organizeCode = :organizeCode and " +
                " mwe.isValid = :isValid and mwe.isDeleted = :isDeleted) ");
        hql.append(" left join MesWorkCenter as mwc on (mwe.workCenterCode = mwc.workCenterCode and mwc.organizeCode = :organizeCode and " +
                " mwc.isValid = :isValid1 and mwc.isDeleted = :isDeleted) ");
        hql.append(" where equ.organizeCode = :organizeCode ");
        hql.append(" and equ.isValid = :isValid1 ");
        hql.append(" and equ.isDeleted = :isDeleted ");
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentCode())) {
            hql.append(" and mwe.equipmentCode like :equipmentCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentName())) {
            hql.append(" and equ.equipmentName like :equipmentName ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            hql.append(" and mwe.workCenterCode = :workCenterCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            hql.append(" and mwe.workCellCode = :workCellCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getAreaCode())) {
            hql.append(" and mwc.areaCode = :areaCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getId())) {
            hql.append(" and mwe.id = :id ");
        }
        Query query = entityManager.createQuery(hql.toString(), Long.class);
        query.setParameter(MesConstWords.ORGANIZE_CODE, mesEquipment.getOrganizeCode());
        query.setParameter(MesConstWords.IS_VALID, mesEquipment.getIsValid());
        query.setParameter("isValid1", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentCode())) {
            query.setParameter("equipmentCode", "%" + mesEquipment.getEquipmentCode() + "%");
        }
        if (!StringUtils.isEmpty(mesEquipment.getEquipmentName())) {
            query.setParameter("equipmentName", "%" + mesEquipment.getEquipmentName() + "%");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            query.setParameter("workCenterCode", mesEquipment.getWorkCenterCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            query.setParameter("workCellCode", mesEquipment.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getAreaCode())) {
            query.setParameter("areaCode", mesEquipment.getAreaCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getId())) {
            query.setParameter("id", mesEquipment.getId());
        }
        return ((Long) query.getSingleResult()).intValue();
    }

    /**
     * 根据生产线，工位获取设备集合
     * @param mesEquipment
     * @return
     */
    @Override
    public List<MesEquipmentModel> findMesEquipmentList(MesEquipment mesEquipment) {
        StringBuffer hql = new StringBuffer();
        hql.append(" select new " + MesEquipmentModel.class.getName() + "(mwe.id as wcId, equ.id, equ.equipmentCode, " +
                " equ.equipmentName,  mwe.workCenterCode, mwe.workCellCode, mwc.areaCode)");
        hql.append(" from MesEquipment as equ ");
        hql.append(" right join MesWcEquipment as mwe on (equ.equipmentCode = mwe.equipmentCode and mwe.organizeCode = :organizeCode and " +
                " mwe.isValid = :isValid and mwe.isDeleted = :isDeleted) ");
        hql.append(" left join MesWorkCenter as mwc on (mwe.workCenterCode = mwc.workCenterCode and mwc.organizeCode = :organizeCode and " +
                " mwc.isValid = :isValid and mwc.isDeleted = :isDeleted) ");
        hql.append(" where equ.organizeCode = :organizeCode ");
        hql.append(" and equ.isValid = :isValid ");
        hql.append(" and equ.isDeleted = :isDeleted ");
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            hql.append(" and mwe.workCellCode = :workCellCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            hql.append(" and mwe.workCenterCode = :workCenterCode ");
        }
        hql.append(" order by equ.modifyDatetime desc ");
        Query query = entityManager.createQuery(hql.toString(), MesEquipmentModel.class);
        query.setParameter(MesConstWords.ORGANIZE_CODE, mesEquipment.getOrganizeCode());
        query.setParameter(MesConstWords.IS_VALID, CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            query.setParameter("workCellCode", mesEquipment.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            query.setParameter("workCenterCode", mesEquipment.getWorkCenterCode());
        }
        return query.getResultList();
    }

    /**
     * 根据条件查询设备所有有效数据-组织模型下拉框及联
     * @param mesEquipment
     * @return
     */
    @Override
    public List<MesEquipmentModel> queryMesEquipmentList(MesEquipment mesEquipment) {
        StringBuffer hql = new StringBuffer();
        hql.append(" select new " + MesEquipmentModel.class.getName() + "(mwe.id, " +
                " concat(equ.equipmentName, '(', mwc1.workCellName, ')') as equipmentNameAndworkCellName, " +
                " equ.equipmentCode, equ.equipmentName, mwc.areaCode, mwe.workCenterCode, mwe.workCellCode)" );
        hql.append(" from MesEquipment as equ ");
        hql.append(" right join MesWcEquipment as mwe on (equ.equipmentCode = mwe.equipmentCode and mwe.organizeCode = :organizeCode and " +
                " mwe.isValid = :isValid and mwe.isDeleted = :isDeleted) ");
        hql.append(" left join MesWorkCenter as mwc on (mwe.workCenterCode = mwc.workCenterCode and mwc.organizeCode = :organizeCode and " +
                " mwc.isValid = :isValid and mwc.isDeleted = :isDeleted) ");
        hql.append(" left join MesWorkCell as mwc1 on (mwe.workCellCode = mwc1.workCellCode and mwc1.organizeCode = :organizeCode and " +
                " mwc1.isValid = :isValid and mwc1.isDeleted = :isDeleted) ");
        hql.append(" where equ.organizeCode = :organizeCode ");
        hql.append(" and equ.isValid = :isValid ");
        hql.append(" and equ.isDeleted = :isDeleted ");
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            hql.append(" and mwe.workCellCode = :workCellCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            hql.append(" and mwe.workCenterCode = :workCenterCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getAreaCode())) {
            hql.append(" and mwc.areaCode = :areaCode ");
        }
        hql.append(" order by equ.modifyDatetime desc ");
        Query query = entityManager.createQuery(hql.toString(), MesEquipmentModel.class);
        query.setParameter(MesConstWords.ORGANIZE_CODE, mesEquipment.getOrganizeCode());
        query.setParameter(MesConstWords.IS_VALID, CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            query.setParameter("workCellCode", mesEquipment.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            query.setParameter("workCenterCode", mesEquipment.getWorkCenterCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getAreaCode())) {
            query.setParameter("areaCode", mesEquipment.getAreaCode());
        }
        return query.getResultList();
    }

    /**
     * 根据条件查询设备所有有效数据-设备作业任务下拉框
     * @param mesEquipment
     * @return
     */
    @Override
    public List<MesEquipmentModel> queryMesEquipmentListByParams(MesEquipment mesEquipment) {
        StringBuffer hql = new StringBuffer();
        hql.append(" select distinct new " + MesEquipmentModel.class.getName() + "(equ.id, equ.equipmentCode, equ.equipmentName, " +
                " mwe.workCenterCode, equ.equipmentCategory, equ.modifyDatetime)" );
        hql.append(" from MesEquipment as equ ");
        hql.append(" left join MesWcEquipment as mwe on (equ.equipmentCode = mwe.equipmentCode and mwe.organizeCode = :organizeCode and " +
                " mwe.isValid = :isValid and mwe.isDeleted = :isDeleted) ");
        hql.append(" left join MesWorkCenter as mwc on (mwe.workCenterCode = mwc.workCenterCode and mwc.organizeCode = :organizeCode and " +
                " mwc.isValid = :isValid and mwc.isDeleted = :isDeleted) ");
        hql.append(" where equ.organizeCode = :organizeCode ");
        hql.append(" and equ.isValid = :isValid ");
        hql.append(" and equ.isDeleted = :isDeleted ");
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            hql.append(" and mwe.workCellCode = :workCellCode ");
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            hql.append(" and mwe.workCenterCode = :workCenterCode ");
        }
        hql.append(" order by equ.modifyDatetime desc ");
        Query query = entityManager.createQuery(hql.toString(), MesEquipmentModel.class);
        query.setParameter(MesConstWords.ORGANIZE_CODE, mesEquipment.getOrganizeCode());
        query.setParameter(MesConstWords.IS_VALID, CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        if (!StringUtils.isEmpty(mesEquipment.getWorkCellCode())) {
            query.setParameter("workCellCode", mesEquipment.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(mesEquipment.getWorkCenterCode())) {
            query.setParameter("workCenterCode", mesEquipment.getWorkCenterCode());
        }
        return query.getResultList();
    }

    /**
     * 查询设备绑定的生产线
     * @param organizeCode
     * @param equipmentCode
     * @return
     */
    @Override
    public List<MesWorkCenter> getEquipmentWorkCenterCode(String organizeCode, String equipmentCode) {
        StringBuffer hql = new StringBuffer();
        hql.append(" select mwc from MesWorkCenter as mwc ");
        hql.append(" left join MesWcEquipment as mwe on (mwc.workCenterCode = mwe.workCenterCode and mwe.organizeCode = :organizeCode and " +
                " mwe.isValid = :isValid and mwe.isDeleted = :isDeleted) ");
        hql.append(" where mwc.organizeCode = :organizeCode and mwe.equipmentCode = :equipmentCode");
        Query query = entityManager.createQuery(hql.toString(), MesWorkCenter.class);
        query.setParameter(MesConstWords.ORGANIZE_CODE, organizeCode);
        query.setParameter(MesConstWords.IS_VALID, CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("equipmentCode", equipmentCode);
        return query.getResultList();
    }

}
