package cn.estsh.i3plus.ext.mes.apiservice.daoimpl;

import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesCraftDao;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesCraft;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;

/**
 * @Description :工艺信息
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/6 15:52
 * @Modify:
 **/
@Service
@Slf4j
public class MesCraftDaoImpl implements IMesCraftDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    @ApiOperation("根据工艺流程代码查询工艺")
    public ListPager<MesCraft> findMesCraft(MesCraft mesCraft, Pager pager) {

        StringBuffer hql = new StringBuffer("from MesCraft mc" +
                " left join MesProcessCraftCfg pc on mc.craftCode = pc.craftCode " +
                " left join MesRouteProcess rp on pc.processCode = rp.processCode " +
                " where mc.isDeleted=:isDeleted and mc.isValid = :isValid and mc.organizeCode = :organizeCode " +
                " and pc.isDeleted=:isDeleted and pc.isValid = :isValid and pc.organizeCode = :organizeCode " +
                " and rp.isDeleted=:isDeleted and rp.isValid = :isValid and rp.organizeCode = :organizeCode ");
        if (!StringUtil.isEmpty(mesCraft.getRouteCode())) {
            hql.append(" and rp.routeCode = :routeCode");
        }
        if (!StringUtil.isEmpty(mesCraft.getCraftCode())) {
            hql.append(" and mc.craftCode = :craftCode");
        }
        if (!StringUtil.isEmpty(mesCraft.getCraftName())) {
            hql.append(" and mc.craftName like '%").append(mesCraft.getCraftName()).append("%' ");
        }
        Query query = entityManager.createQuery("select mc " + hql, MesCraft.class);
        Query countQuery = entityManager.createQuery("select count(1) " + hql, Long.class);
        query.setParameter("organizeCode", mesCraft.getOrganizeCode());
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());

        countQuery.setParameter("organizeCode", mesCraft.getOrganizeCode());
        countQuery.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        countQuery.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        if (!StringUtil.isEmpty(mesCraft.getRouteCode())) {
            query.setParameter("routeCode", mesCraft.getRouteCode());
            countQuery.setParameter("routeCode", mesCraft.getRouteCode());
        }
        if (!StringUtil.isEmpty(mesCraft.getCraftCode())) {
            query.setParameter("craftCode", mesCraft.getCraftCode());
            countQuery.setParameter("craftCode", mesCraft.getCraftCode());
        }
        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        pager = PagerHelper.getPager(pager, count);
        if (count <= 0) {
            return new ListPager<>(new ArrayList<>(), pager);
        }
        query.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize());
        return new ListPager<>(query.getResultList(), pager);
    }
}
