package cn.estsh.i3plus.ext.mes.apiservice.daoimpl;

import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesDefectRecordDao;
import cn.estsh.i3plus.ext.mes.pojo.model.MesDefectRecordModel;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description :SAP生产计划
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/6 15:52
 * @Modify:
 **/
@Service
@Slf4j
public class MesDefectRecordDaoImpl implements IMesDefectRecordDao {


    @Autowired
    private EntityManager entityManager;


    @Override
    public List<MesDefectRecordModel> queryMesDefectRecordGroupBy(String organizeCode, Integer count, String createDateTimeStart, String createDateTimeEnd) {

        StringBuffer hql = new StringBuffer("select mpi.part_no, mp.part_type_code,mpid.front_back, mpid.defect_location, mpid.defect_type_code ");
        hql.append(" from mes_part_inspection_detail as mpid");
        hql.append(" INNER JOIN mes_part_inspection AS mpi ON mpid.pid = mpi.id   ");
        hql.append(" and mpid.organize_code = mpi.organize_code  and mpid.is_deleted = mpi.is_deleted and mpid.is_valid = mpi.is_valid ");
        hql.append(" INNER JOIN mes_part AS mp ON mp.part_no = mpi.part_no    ");
        hql.append(" and mp.organize_code = mpi.organize_code  and mp.is_deleted = mpi.is_deleted and mp.is_valid = mpi.is_valid ");
        hql.append(" where mpid.organize_code = :organizeCode ");
        hql.append(" and mpid.is_deleted = :isDeleted ");
        hql.append(" and mpid.is_valid = :isValid ");
        hql.append(" and mpid.create_date_time >= :createDateTimeStart");
        hql.append(" and mpid.create_date_time <= :createDateTimeEnd");
        hql.append(" GROUP BY mpi.part_no, mp.part_type_code, mpid.front_back, mpid.defect_location, mpid.defect_type_code ");
        hql.append(" having count(1) >= " + count);

        Query query = entityManager.createNativeQuery(hql.toString());
        query.setParameter("organizeCode", organizeCode);
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("createDateTimeStart", createDateTimeStart);
        query.setParameter("createDateTimeEnd", createDateTimeEnd);

        List list = query.getResultList();

        List<MesDefectRecordModel> modelList = new ArrayList<>();

        for (Object result : list) {
            Object[] cells = (Object[]) result;
            MesDefectRecordModel model = new MesDefectRecordModel();
            String partNo = String.valueOf(cells[0]);
            String partTypeCode = String.valueOf(cells[1]);
            String sides = String.valueOf(cells[2]);
            String defectLocationCode = String.valueOf(cells[3]);
            String defectCode = String.valueOf(cells[4]);
            model.setPartNo(partNo);
            model.setPartTypeCode(partTypeCode);
            model.setFrontBack(Integer.parseInt(sides));
            model.setDefectLocation(defectLocationCode);
            model.setDefectTypeCode(defectCode);
            modelList.add(model);
        }

        return modelList;

    }
}
