package cn.estsh.i3plus.ext.mes.apiservice.daoimpl;

import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesProductPlanDao;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesProductPlan;
import cn.estsh.i3plus.pojo.mes.bean.MesProductPlan;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @Description :SAP生产计划
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/6 15:52
 * @Modify:
 **/
@Service
@Slf4j
public class MesProductPlanDaoImpl implements IMesProductPlanDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public ListPager<MesProductPlan> queryMesProductPlanPager(MesProductPlan mesProductPlan, Pager pager) {
        //查询数据
        StringBuffer dataHql = new StringBuffer("select sap.plan_order_no,sap.plan_part_no,ifnull(sap.plan_qty,0) as sumQty," +
                "ifnull(mes.reported_qty,0) as completeQty,sap.unit,sap.plan_start_date,sap.plan_end_date,sap.part_mapping_work_center_code," +
                "sap.plan_organize_code,sap.create_date_time,sap.create_user,sap.modify_date_time,sap.modify_user");
        StringBuilder hql = new StringBuilder();
        hql.append(" from mes_plan_sap sap left join mes_work_order mes on sap.plan_order_no = mes.plan_order_no and sap.plan_part_no = mes.part_no " +
                "where sap.is_deleted=:isDeleted and sap.is_valid=:isValid and sap.organize_code=:organizeCode " +
                " and ((mes.is_deleted=:isDeleted and mes.is_valid=:isValid and mes.organize_code=:organizeCode) or mes.id is null)");
        //拼接查询条件
        packWhere(mesProductPlan, hql);
        StringBuffer unionDataHql = new StringBuffer("select sap.plan_order_no,mes.part_no,ifnull(mes.qty,0) as sumQty," +
                "ifnull(mes.reported_qty,0) as completeQty,part.unit,sap.plan_start_date,sap.plan_end_date,sap.part_mapping_work_center_code," +
                "sap.plan_organize_code,mes.create_date_time,mes.create_user,mes.modify_date_time,mes.modify_user");
        StringBuilder unionHql = new StringBuilder();
        unionHql.append(" from mes_plan_sap sap right join mes_work_order mes on sap.plan_order_no = mes.plan_order_no and sap.plan_part_no = mes.part_no " +
                " left join mes_part part on part.part_no = mes.part_no " +
                "where mes.is_deleted=:isDeleted and mes.is_valid=:isValid and mes.organize_code=:organizeCode and sap.id is null ");
        //拼接查询条件
        packWhere(mesProductPlan, unionHql);
        //查询行数
        int count = getCount(mesProductPlan, hql) + getCount(mesProductPlan, unionHql);
        pager = PagerHelper.getPager(pager, count);
        if (count <= 0) {
            return new ListPager<>(new ArrayList<>(), pager);
        }
        //排序
        String orderBy = " order by create_date_time desc";
        //查询数据
        Query query = entityManager.createNativeQuery(dataHql.append(hql) + " union all " + unionDataHql.append(unionHql) + orderBy);
        //赋值
        setParameter(mesProductPlan, query);
        List resultList = query.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize()).getResultList();
        List<MesProductPlan> list = new ArrayList<>();
        for (Object result : resultList) {
            Object[] cells = (Object[]) result;
            MesProductPlan tjMgnBoardModel = new MesProductPlan(
                    Objects.isNull(cells[0])?"":String.valueOf(cells[0]),
                    Objects.isNull(cells[1])?"":String.valueOf(cells[1]),
                    Objects.isNull(cells[2])?0:Double.parseDouble(String.valueOf(cells[2])),
                    Objects.isNull(cells[3])?0:Double.parseDouble(String.valueOf(cells[3])),
                    Objects.isNull(cells[4])?"":String.valueOf(cells[4]),
                    Objects.isNull(cells[5])?"":String.valueOf(cells[5]),
                    Objects.isNull(cells[6])?"":String.valueOf(cells[6]),
                    Objects.isNull(cells[7])?"":String.valueOf(cells[7]),
                    Objects.isNull(cells[8])?"":String.valueOf(cells[8]),
                    Objects.isNull(cells[9])?"":String.valueOf(cells[9]),
                    Objects.isNull(cells[10])?"":String.valueOf(cells[10]),
                    Objects.isNull(cells[11])?"":String.valueOf(cells[11]),
                    Objects.isNull(cells[12])?"":String.valueOf(cells[12]));
            list.add(tjMgnBoardModel);
        }
        return new ListPager<>(list, pager);
    }

    private int getCount(MesProductPlan mesProductPlan, StringBuilder hql) {
        Query query = entityManager.createNativeQuery("select count(1) " + hql);
        //赋值
        setParameter(mesProductPlan, query);
        return Integer.parseInt(query.getSingleResult() + "");
    }

    private void setParameter(MesProductPlan mesProductPlan, Query query) {
        query.setParameter("organizeCode", mesProductPlan.getOrganizeCode());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        if (!StringUtil.isEmpty(mesProductPlan.getPlanOrderNo())) {
            query.setParameter("planOrderNo", mesProductPlan.getPlanOrderNo());
        }
        //计划物料
        if (!StringUtil.isEmpty(mesProductPlan.getPlanPartNo())) {
            query.setParameter("planPartNo", mesProductPlan.getPlanPartNo());
        }
        //生产线
        if (!StringUtil.isEmpty(mesProductPlan.getPartMappingWorkCenterCode())) {
            query.setParameter("partMappingWorkCenterCode", mesProductPlan.getPartMappingWorkCenterCode());
        }
        //计划开始&结束
        if (!StringUtil.isEmpty(mesProductPlan.getPlanStartDate())) {
            query.setParameter("planStartDate", mesProductPlan.getPlanStartDate());
        }
        if (!StringUtil.isEmpty(mesProductPlan.getPlanEndDate())) {
            query.setParameter("planEndDate", mesProductPlan.getPlanEndDate());
        }
        //开始创建&结束日期
        if (!StringUtil.isEmpty(mesProductPlan.getCreateDateTimeStart())) {
            query.setParameter("createDateTimeStart", mesProductPlan.getCreateDateTimeStart());
        }
        if (!StringUtil.isEmpty(mesProductPlan.getCreateDateTimeEnd())) {
            query.setParameter("createDateTimeEnd", mesProductPlan.getCreateDateTimeEnd());
        }
    }

    private void packWhere(MesProductPlan mesProductPlan, StringBuilder hql) {
        //计划订单号
        if (!StringUtil.isEmpty(mesProductPlan.getPlanOrderNo())) {
            hql.append(" and sap.plan_order_no = :planOrderNo");
        }
        //计划物料
        if (!StringUtil.isEmpty(mesProductPlan.getPlanPartNo())) {
            hql.append(" and sap.plan_part_no = :planPartNo");
        }
        //生产线
        if (!StringUtil.isEmpty(mesProductPlan.getPartMappingWorkCenterCode())) {
            hql.append(" and sap.part_mapping_work_center_code = :partMappingWorkCenterCode");
        }
        //计划开始&结束
        if (!StringUtil.isEmpty(mesProductPlan.getPlanStartDate())) {
            hql.append(" and sap.plan_start_date >= :planStartDate");
        }
        if (!StringUtil.isEmpty(mesProductPlan.getPlanEndDate())) {
            hql.append(" and sap.plan_end_date <= :planEndDate");
        }
        //开始创建&结束日期
        if (!StringUtil.isEmpty(mesProductPlan.getCreateDateTimeStart())) {
            hql.append(" and sap.create_date_time >= :createDateTimeStart");
        }
        if (!StringUtil.isEmpty(mesProductPlan.getCreateDateTimeEnd())) {
            hql.append(" and sap.create_date_time <= :createDateTimeEnd");
        }
    }

}
