package cn.estsh.i3plus.ext.mes.apiservice.dbinterface;

import cn.estsh.i3plus.ext.mes.pojo.util.CronUtil;
import cn.estsh.i3plus.ext.mes.pojo.util.OverwriteStringJoin;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy;
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
/**
 * @Description :
 * @Reference :
 * @Author : gsz
 * @CreateDate : 2024-05-10 16:08
 * @Modify:
 **/
public class MesSAPDbDataMapper extends WmsSAPDbSyncBase {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesSAPDbDataMapper.class);

    private DynamicDataSourceProxy mesDataSourceProxy;

    @Getter
    @Setter
    private Connection srcConn;
 
    public MesSAPDbDataMapper(DynamicDataSourceProxy mesDataSourceProxy) {
        this.mesDataSourceProxy = mesDataSourceProxy;
    }

    /**
     * 查询 SQL 返回数据转换配置对象的列表
     * @return
     * @throws Exception
     */
    public List<MesInterfaceDataMapper> readDataMappersBySQL(String groupName, String id,String organizeCode,List<String> destBeanNameList,List<String> destBeanNameNotList) throws Exception {
        String sqlString = "select * from mes_interface_data_mapper " +
                "where is_deleted=2 and is_valid=1 and group_name='"
                + groupName + "' ";

        // 如果 id 不为空，则按照单个 id 查询
        if (id != null && !id.isEmpty()) {
            // 带 id 调试时不考虑删除和有效2个字段
            sqlString = sqlString.replace("is_deleted=2 and is_valid=1 and", " ");
            sqlString = sqlString + " and id=" + id;
        }

        // 如果 organizeCode 不为空，则按照 organizeCode 查询
        if (organizeCode != null && !organizeCode.isEmpty()) {
            // 带 id 调试时不考虑删除和有效2个字段
            sqlString = sqlString + " and organize_code= '" + organizeCode +"'";
        }
        // 按照目标表进行拆分，参数不为空则in
        if (!CollectionUtils.isEmpty(destBeanNameList)) {
            sqlString = sqlString + " and dest_bean_name in ("+ OverwriteStringJoin.join(",",destBeanNameList)+")";
            LOGGER.info("接口映射添加目标表参数，sql为:{}",sqlString);
        }
        // 按照目标表进行拆分，参数不为空则 not in
        if (!CollectionUtils.isEmpty(destBeanNameNotList)) {
            sqlString = sqlString + " and dest_bean_name not in ("+ OverwriteStringJoin.join(",",destBeanNameNotList)+")";
            LOGGER.info("接口映射不添加目标表参数，sql为:{}",sqlString);
        }

        sqlString = sqlString + " order by seq";

        List<Map<String, Object>> rows = mesDataSourceProxy.queryMapList(sqlString, srcConn);

        if (rows == null || rows.size() == 0) {
            LOGGER.error("not set data in talbe: mes_interface_data_mapper !");
            throw new IllegalArgumentException("not set data in talbe: mes_interface_data_mapper !");
        }

        List<MesInterfaceDataMapper> dataMappers = new ArrayList<>();

        for(Map<String, Object> row : rows) {
            MesInterfaceDataMapper mapper = new MesInterfaceDataMapper();

            mapper.setSeq(this.getSafetyInteger(row.get("seq")));
            mapper.setOrganizeCode(this.getSafetyString(row.get("organize_code")));
            mapper.setSrcGetLimit(this.getSafetyInteger(row.get("src_get_limit")));
            mapper.setDataSource(this.getSafetyString(row.get("src_data_source")));
            mapper.setSrcTableName(this.getSafetyString(row.get("src_table_name")));
            mapper.setSrcOrderBy(this.getSafetyString(row.get("src_order_by")));
            mapper.setSrcWhere(this.getSafetyString(row.get("src_where")));
            mapper.setSrcGroupColumns(this.getSafetyString(row.get("src_group_columns")));
            mapper.setSrcUpdateSync(this.getSafetyString(row.get("src_update_sync")));
            mapper.setSrcPkColumns(this.getSafetyString(row.get("src_pk_columns")));
            mapper.setCopyByOrgs(this.getSafetyString(row.get("copy_py_orgs")));
            mapper.setDestBeanName(this.getSafetyString(row.get("dest_bean_name")));
            mapper.setDestPkProperties(this.getSafetyString(row.get("dest_pk_properties")));
            mapper.setDestColumnMapping(this.getSafetyString(row.get("dest_column_mapping")));
            mapper.setBusiRoute(this.getSafetyString(row.get("busi_route")));
            mapper.setId(this.getSafetyLong(row.get("id")));
            mapper.setCopyByOrgs(this.getSafetyString(row.get("copy_by_orgs")));
            mapper.setSyncCron(this.getSafetyString(row.get("sync_cron")));
            mapper.setNextSyncTime(this.getSafetyString(row.get("next_sync_time")));

            mapper.setSyncFrequency(this.getSafetyInteger(row.get("sync_frequency")));
            mapper.setLastSyncTime(this.getSafetyString(row.get("last_sync_time")));
            mapper.setSoftDataType(this.getSafetyInteger(row.get("soft_data_type")));

            // 与软适配和脚本对接相关的属性
            mapper.softAdaptorCode = this.getSafetyString(row.get("soft_adapator_code"));
            mapper.scriptNo = this.getSafetyString(row.get("script_no"));
            mapper.useScriptPull = this.getSafetyInteger(row.get("use_script_pull"));
            mapper.useScriptPush = this.getSafetyInteger(row.get("use_script_push"));
            mapper.useScriptMark = this.getSafetyInteger(row.get("use_script_mark"));
            mapper.useScriptFilter = this.getSafetyInteger(row.get("use_script_filter"));

            // 备份表名
            mapper.setCutToTableName(this.getSafetyString(row.get("cut_to_table_name")));

            dataMappers.add(mapper);
        }

        return dataMappers;
    }

    public void updateWmsInterfaceDataMapperLastSyncTimeById(Long id) throws SQLException {
        StringBuffer sql = new StringBuffer("update mes_interface_data_mapper");
        sql.append(" set last_sync_time='" + TimeTool.getNowTime(true) + "'");
        sql.append(" where id=" + id);
        mesDataSourceProxy.execute(sql.toString());
    }

    /**
     * 更新接口的下次执行时间
     * @param mapper
     * @throws SQLException
     */
    public void updateNextSyncTime(MesInterfaceDataMapper mapper) throws SQLException {
        if (mapper != null && StringUtils.isNotEmpty(mapper.getSyncCron())
                && StringUtils.isNotEmpty(mapper.getNextSyncTime())) {
            // 计算下次执行时间
            String nextSyncTime = CronUtil.getRecentTriggerTime(mapper.getSyncCron());
            String sqlString = String.format("update mes_interface_data_mapper set next_sync_time='%s' where id=%s",
                    nextSyncTime, mapper.getId());
            mesDataSourceProxy.execute(sqlString);
            LOGGER.debug("接口ID {} 名称 {} 下次执行时间更新为 {}", mapper.getId(), mapper.getInterfaceName(), nextSyncTime);
        }
    }

    /**
     * 重写判断对象相等的方法
     * @param obj
     * @return
     */
    @Override
    public boolean equals(Object obj) {
        return super.equals(obj);
    }

    /**
     * 重写 hashCode 方法
     * @return
     */
    @Override
    public int hashCode() {
        return super.hashCode();
    }
}
