package cn.estsh.i3plus.ext.mes.apiservice.dbinterface;

import cn.estsh.i3plus.mes.apiservice.serviceimpl.engine.script.EngineScriptManager;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import lombok.Getter;
import lombok.Setter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description :
 * @Reference :
 * @Author : gsz
 * @CreateDate : 2024-05-10 16:08
 * @Modify:
 **/
@Component
public class WmsSAPDbExpression {
    /**
     * 获取系统相关的 ID
     */
    @Autowired
    private WmsSAPDbID wmsSAPDbID;

    /**
     * 脚本引擎
     */
    @Autowired
    private EngineScriptManager engineScriptManager;

    @Getter
    @Setter
    private Connection srcConn;

    WmsSAPDbQuery wmsSAPDbQuery = null;

    // 更新时不处理的特殊标志
    public static final String NOT_TRANS = "NotTrans";

    /**
     * 执行表达式计算
     * @param expString 表达式
     * @param varMap 临时变量
     * @throws Exception
     */
    public Object parse(String expString, Map<String, Object> varMap) {
        ExpressionParser parser = new SpelExpressionParser();

        StandardEvaluationContext ctx = new StandardEvaluationContext();

        // 先设置基础变量
        ctx.setVariables(getConstVarMap());

        // 再设置临时变量
        if (varMap != null) {
            ctx.setVariables(varMap);
        }

        return parser.parseExpression(expString).getValue(ctx);
    }

    /**
     * 执行表达式计算，不带变量
     * @param expString
     * @return
     * @throws Exception
     */
    public Object parse(String expString) {
        return parse(expString, null);
    }

    /**
     * 初始化基础变量
     */
    public Map<String, Object> getConstVarMap() {
        if (wmsSAPDbQuery == null) {
            wmsSAPDbQuery = new WmsSAPDbQuery();
            // 初始化查询对象
            wmsSAPDbQuery.setSrcConn(srcConn);
            // new 的对象需要手工注入 bean
            if (SpringContextsUtil.getApplicationContext() != null) {
                SpringContextsUtil.getApplicationContext().
                        getAutowireCapableBeanFactory().autowireBean(wmsSAPDbQuery);
            }
        }

        Map<String, Object> constVarMap = new HashMap<>();

        constVarMap.put("date", getFormatDate("yyyy-MM-dd"));
        constVarMap.put("time", getFormatDate("HH:mm:ss"));
        constVarMap.put("sap_date", getFormatDate("yyyyMMdd"));
        constVarMap.put("sap_time", getFormatDate("HHmmss"));
        constVarMap.put("now", getFormatDate("yyyy-MM-dd HH:mm:ss"));
        constVarMap.put("Q", wmsSAPDbQuery);
        constVarMap.put("ID", wmsSAPDbID);
        constVarMap.put("I", wmsSAPDbID);
        constVarMap.put("S", engineScriptManager);

        return constVarMap;
    }

    /**
     * 获取格式化后的日期字符串
     * @param format
     * @return
     */
    public static String getFormatDate(String format) {
        DateFormat df = new SimpleDateFormat(format);
        return df.format(new Date());
    }
}
