package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;

import cn.estsh.i3plus.ext.mes.api.base.IMesBtoJobService;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
/*
入参
{
	"organizeCode": "CK01",
	"partGroupCode": [
		"CK01-Code",
		"CK02-Code"
	]

}
 */
/**
 * @Description : BTO生成工单
 * @Reference :
 * @Author : castle
 * @CreateDate : 2024-05-10 11:03
 * @Modify:
 **/
// 禁止 JOB 并发执行
@Component
@ApiOperation("Bto-job")
public class MesBtoJob extends BaseMesScheduleJob {
    public static final Logger LOGGER = LoggerFactory.getLogger(MesBtoJob.class);

    @Autowired
    private IMesBtoJobService btoJobService;

    public MesBtoJob() {
        super(MesBtoJob.class, "BTO生成工单");
        this.setMultiInstance(true);
    }

    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        try {
            String jobParam = this.getJobParam();

            if (StringUtils.isBlank(jobParam)) {
                throw new IllegalArgumentException("jobc参数为空，请检查参数");
            }
            Map<String, Object> paramMap;
            try {
                paramMap = (Map<String, Object>) JSONObject.parse(jobParam);
            } catch (Exception e) {
                LOGGER.error("参数格式不是JSON");
                return;

            }
            if (CollectionUtils.isEmpty(paramMap)) {
                LOGGER.error("BTO生成工单,没有配置参数");
                return;
            }
            String organizeCode = (String) paramMap.get("organizeCode");
            List<String> groupCodeList = (List<String>) paramMap.get("groupCode");
            btoJobService.doCreateBto(organizeCode, groupCodeList);
            LOGGER.info("缺陷告警配置job结束 ----- end");

        } catch (Exception e) {
            LOGGER.error("SAP接口表数据同步作业任务结束e:{}", e.toString());
        }
    }

}
