package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;

import cn.estsh.i3plus.ext.mes.apiservice.dbinterface.MesSAPDbAdapter;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceEnumUtil;
import cn.estsh.i3plus.pojo.model.wms.WmsJobParamModel;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @Description : 按工厂划分：把 SAP 接口表的数据同步到 MES 业务表中（单据+基础数据）
 * @Reference :
 * @Author : gsz
 * @CreateDate : 2024-05-10 11:03
 * @Modify:
 **/
// 禁止 JOB 并发执行
@DisallowConcurrentExecution
@ApiOperation("SAP接口表数据同步作业按工厂划分")
public class MesSAPDbTransByWorksJob extends BaseMesScheduleJob {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesSAPDbTransByWorksJob.class);

    private static final long serialVersionUID = 9072058154235836696L;

    @Value("${server.port}")
    private String serverPort;
    @Value("${impp.server.ip}")
    private String ip;

    public MesSAPDbTransByWorksJob() {
        super(MesSAPDbTransByWorksJob.class, "SAP接口表数据同步作业按工厂划分");
        this.setMultiInstance(true);
    }

    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        try {
            String jobParam = this.getJobParam();

            if (StringUtils.isBlank(jobParam)) {
                throw new IllegalArgumentException("jobc参数为空，请检查参数");
            }

            //参数： [{ "organizeCodeList" : [ "3610",2752","2751"],"groupName" : "SAP2MES"}]
            //参数： [{ "organizeCodeList" : [ "3610",2752","2751"],"groupName" : "WMS2SAP"}]
            List<WmsJobParamModel> wmsJobParamModelList = JsonUtilTool.toList(jobParam, WmsJobParamModel.class);

            if (!CollectionUtils.isEmpty(wmsJobParamModelList)) {

                //正常应该一个工厂一个分组JOB。所以一个【jobParam】就有一个解析对象。
                WmsJobParamModel wmsJobParamModel = wmsJobParamModelList.get(0);

                //解析job参数
                List<String> organizeCodeList = wmsJobParamModel.getOrganizeCodeList();
                List<String> destBeanNameList = wmsJobParamModel.getDestBeanNameList();
                List<String> destBeanNameNotList = wmsJobParamModel.getDestBeanNameNotList();
                String groupName = wmsJobParamModel.getGroupName();

                // 判断作业参数是否支持
                if (MesInterfaceEnumUtil.DIRECTION_TYPE.nameOf(groupName) == -1) {
                    throw new IllegalArgumentException("不支持的作业参数：" + groupName);
                }

                LOGGER.info("SAP接口表数据同步作业按工厂划分开始 -----start");

                // 开始同步数据
                // 一定要 new 对象，不然会有并发问题
                MesSAPDbAdapter mesSAPDbAdapter;
                for (String organizeCode : organizeCodeList) {

                    mesSAPDbAdapter = new MesSAPDbAdapter();
                    // new 的对象需要手工注入 bean
                    if (SpringContextsUtil.getApplicationContext() != null) {
                        SpringContextsUtil.getApplicationContext().
                                getAutowireCapableBeanFactory().autowireBean(mesSAPDbAdapter);
                    }
                    mesSAPDbAdapter.syncDataByOrganizeCode(groupName, organizeCode,destBeanNameList,destBeanNameNotList);
                }

//                LOGGER.info("SAP接口表数据同步作业按工厂划分开始结束 -----  projectName：{}, port:{}",
//                        applicationProperties.getApplicationName(), applicationProperties.getServerPort());
                LOGGER.info("SAP接口表数据同步作业按工厂划分开始结束 ----- end");
            }

        } catch (Exception e) {
            LOGGER.error("SAP接口表数据同步作业任务结束e:{}",e.toString());
            //sendErrorMessage(e.toString());
        }
    }

}
