package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;

import cn.estsh.i3plus.ext.mes.api.base.IMesCcscTaskService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderService;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @Description : MES根据工单生成CCSC检测任务给SAP
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/5/23 9:58
 * @Modify:
 **/
// 禁止 JOB 并发执行
@DisallowConcurrentExecution
@ApiOperation("MES根据工单生成CCSC检测任务给SAP")
public class MesWorkOrderProcessCcscTaskToSapJob extends BaseMesScheduleJob {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesWorkOrderProcessCcscTaskToSapJob.class);

    private static final long serialVersionUID = 5397182238074237383L;

    @Autowired
    private IMesWorkOrderService mesWorkOrderService;

    @Autowired
    private IMesCcscTaskService mesCcscTaskService;

    @Value("${server.port}")
    private String serverPort;
    @Value("${impp.server.ip}")
    private String ip;

    public MesWorkOrderProcessCcscTaskToSapJob() {
        super(MesWorkOrderProcessCcscTaskToSapJob.class, "汇总工单给SAP");
        this.setMultiInstance(true);
    }

    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        try {
            String jobParam = this.getJobParam();

            if (StringUtils.isBlank(jobParam)) {
                throw new IllegalArgumentException("job参数为空，请检查参数");
            }
            String[] organizeCodeList = jobParam.split(",");
            LOGGER.info("汇总工单给SAP开始 -----start");

            for (String organizeCode : organizeCodeList) {
                List<MesWorkOrder> mesWorkOrderList = mesWorkOrderService.queryMesWorkOrderProcessToSap(organizeCode);
                if (!CollectionUtils.isEmpty(mesWorkOrderList)){
                    mesCcscTaskService.doCcscTaskByMesWorkOrder(organizeCode,mesWorkOrderList);
                }
            }
            LOGGER.info("汇总工单给SAP开始结束 ----- end");

        } catch (Exception e) {
            LOGGER.error("汇总工单给SAP任务结束e:{}",e.toString());
        }
    }

}
