package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCarSeriesService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCarSeries;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class MesCarSeriesService extends BaseMesService<MesCarSeries> implements IMesCarSeriesService {


    protected void onInsertBean(MesCarSeries item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCarSeriesCode(), "车系代码不能为空");
        ValidatorBean.checkNotNull(item.getCarSeriesName(), "车系名称不能为空");
        ValidatorBean.checkNotNull(item.getCustCode(), "客户代码不能为空");
        ValidatorBean.checkNotNull(item.getCustOrganizeCode(), "客户工厂代码不能为空");


        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCarSeriesCode(), "carSeriesCode", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getCustCode(), "custCode", seriesPackBean);
        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】车系代码【%s】客户代码已经存在，请检查数据", item.getCarSeriesCode(), item.getCustCode())
                    .build();
        }

    }

    protected void onUpdateBean(MesCarSeries item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCarSeriesCode(), "车系代码不能为空");
        ValidatorBean.checkNotNull(item.getCarSeriesName(), "车系名称不能为空");
        ValidatorBean.checkNotNull(item.getCustCode(), "客户代码不能为空");
        ValidatorBean.checkNotNull(item.getCustOrganizeCode(), "客户工厂代码不能为空");

        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCarSeriesCode(), "carSeriesCode", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(item.getCustCode(), "custCode", seriesPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", seriesPackBean);
        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】车系代码【%s】客户代码已经存在，请检查数据", item.getCarSeriesCode(), item.getCustCode())
                    .build();
        }
    }

    protected void setPackQueryBean(MesCarSeries bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getCarSeriesCode(), "carSeriesCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCarSeriesName(), "carSeriesName", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustCode(), "custCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustOrganizeCode(), "custOrganizeCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getYearModel(), "yearModel", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCustCarSerialCode(), "custCarSerialCode", packBean);
    }

}
