package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCcscTaskService;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pojo.util.CronUtil;
import cn.estsh.i3plus.icloud.core.sdk.ICoreUtilCloud;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.WmsConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.MesCcscRuleRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesCcscTaskRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesCcscTaskServiceImpl extends BaseMesService<MesCcscTask> implements IMesCcscTaskService {
    public static final Logger LOGGER = LoggerFactory.getLogger(MesCcscTaskServiceImpl.class);

    @Autowired
    private MesCcscRuleRepository ccscRuleRDao;
    @Autowired
    private MesCcscTaskRepository ccscTaskRDao;
    @Autowired
    private ICoreUtilCloud sysOrderNoRuleCloud;

    @Override
    protected void setPackQueryBean(MesCcscTask bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getTaskNo(), "taskNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkOrderNo(), "workOrderNo", packBean);
    }

    @Override
    public MesCcscTask insert(MesWorkOrder mesWorkOrder, String orderNo, Integer frequency, Integer rule, Integer qtyRule) {
        MesCcscTask mesCcscTask = new MesCcscTask(mesWorkOrder);
        mesCcscTask.setTaskNo(orderNo);
        mesCcscTask.setFrequency(frequency);
        mesCcscTask.setRule(rule);
        mesCcscTask.setQty((double)qtyRule);
        ConvertBean.serviceModelInitialize(mesCcscTask, mesWorkOrder.getModifyUser());

        return baseRDao.insert(mesCcscTask);
    }


    @Override
    public void doCcscTaskByMesWorkOrder(String organizeCode, List<MesWorkOrder> mesWorkOrderList) {
        //查ccsc规则表
        LOGGER.info("-------- doCcscTaskByMesWorkOrder---{}",organizeCode);
        DdlPackBean ddlPackBeanRule = DdlPackBean.getDdlPackBean(organizeCode);
        List<MesCcscRule> ccscRuleList = ccscRuleRDao.findByHqlWhere(ddlPackBeanRule);
        Map<String, List<MesCcscRule>> ccscRulePartMap = ccscRuleList.stream().collect(Collectors.groupingBy(MesCcscRule::getPartNo));
        Map<String, List<MesWorkOrder>> workOrderPartMap = mesWorkOrderList.stream().collect(Collectors.groupingBy(MesWorkOrder::getPartNo));
        //根据物料对工单表分组
        for (Map.Entry<String, List<MesWorkOrder>> stringListEntry : workOrderPartMap.entrySet()) {
            String key = stringListEntry.getKey();
            List<MesWorkOrder> mesWorkOrders = stringListEntry.getValue();
            MesWorkOrder mesWorkOrder = mesWorkOrders.iterator().next();
            //每一个加工的工单 1.查QMS同步过来的质量ccsc检查规则表
            String nowTime = TimeTool.getNowTime(true);

            if (CollectionUtils.isEmpty(ccscRulePartMap.get(mesWorkOrder.getPartNo()))) {
                continue;
            }
            LOGGER.info("-------- doCcscTaskByMesWorkOrder--80-{}",ccscRulePartMap.get(mesWorkOrder.getPartNo()));
            MesCcscRule ccscRule = ccscRulePartMap.get(mesWorkOrder.getPartNo()).iterator().next();
            if (nowTime.compareTo(ccscRule.getNextExeTime()) > 0) {

                Integer frequency = ccscRule.getFrequency();
                Integer rule = ccscRule.getRule();
                Integer qtyRule = ccscRule.getQty();
                LOGGER.info("-------- doCcscTaskByMesWorkOrder--83-{}",rule);
                List<MesWorkOrder> mesWorkOrderToTask=new ArrayList<>();
                //首件取第一个工单？还是全部工单都同步，取工单明细条码的首件条码？
                switch (rule) {
                    case 1:
                        MesWorkOrder workOrder = mesWorkOrders.stream().sorted(Comparator.comparing(MesWorkOrder::getStartTime)).collect(Collectors.toList()).iterator().next();
                        mesWorkOrderToTask.add(workOrder);
                        break;
                    case 2:
                        mesWorkOrderToTask.add(mesWorkOrders.stream().sorted(Comparator.comparing(MesWorkOrder::getStartTime).reversed()).collect(Collectors.toList()).iterator().next());
                        break;
                    case 3://首尾件
                        mesWorkOrderToTask.add(mesWorkOrders.stream().sorted(Comparator.comparing(MesWorkOrder::getStartTime)).collect(Collectors.toList()).iterator().next());
                        MesWorkOrder next = mesWorkOrders.stream().sorted(Comparator.comparing(MesWorkOrder::getStartTime).reversed()).collect(Collectors.toList()).iterator().next();
                        if (!mesWorkOrderToTask.contains(next)) {
                            mesWorkOrderToTask.add(next);
                        }
                        break;
                    case 4://随机
                        int qty= mesWorkOrders.size()/qtyRule ;
                        mesWorkOrderToTask.addAll(mesWorkOrders.stream().sorted(Comparator.comparing(MesWorkOrder::getStartTime)).collect(Collectors.toList()).subList(0,qty));
                        break;
                    default:
                        break;
                }
                LOGGER.info("-------- doCcscTaskByMesWorkOrder--105-");
                String orderNo;
                //新增质量检验任务 MesCcscTask 工单+物料查询发送Sap的数据
                DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(mesWorkOrder.getWorkOrderNo(), "workOrderNo", ddlPackBean);
                DdlPreparedPack.getStringEqualPack(mesWorkOrder.getPartNo(), "partNo", ddlPackBean);
                List<MesCcscTask> mesCcscTaskList = baseRDao.findByHqlWhere(ddlPackBean);
                if (CollectionUtils.isEmpty(mesCcscTaskList)) {
                    //全部新增task表
                    for (MesWorkOrder workOrder : mesWorkOrderToTask) {
                        //调用平台生成拉动单号流水 CCSC_TASK_NO TASKyyyyMMdd00001
                        BaseResultBean<LinkedHashMap> orderResutlBean = sysOrderNoRuleCloud.getOrderNo(MesCommonConstant.CCSC_TASK_NO);
                        if (orderResutlBean != null && orderResutlBean.isSuccess()) {
                            orderNo = orderResutlBean.getResultObject().get(WmsConstWords.CORE_ORDER_NO).toString();
                        } else {
                            throw ImppExceptionBuilder.newInstance()
                                    .setSystemID(CommonEnumUtil.SOFT_TYPE.WMS.getCode())
                                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                                    .setErrorDetail("生成单号失败")
                                    .setErrorSolution("【CCSC_TASK_NO】请重新操作或查看服务！")
                                    .build();
                        }
                        LOGGER.info("-------- doCcscTaskByMesWorkOrder--129-insert");
                        insert(workOrder, orderNo, frequency, rule,qtyRule);
                    }
                }
                LOGGER.info("-------- doCcscTaskByMesWorkOrder--133-ccscRuleRDao.save");
                //按照班次更新下次执行时间
                ccscRule.setNextExeTime(CronUtil.getRecentTriggerTime(ccscRule.getCron()));
                //设置末次执行时间
                ccscRule.setLastExeTime(nowTime);
                ConvertBean.serviceModelUpdate(ccscRule, mesWorkOrder.getModifyUser());
                ccscRuleRDao.save(ccscRule);
            }
        }
    }
}
