package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCraftService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesCraftDao;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCraft;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @Description : 工艺信息
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/11 13:55
 * @Modify:
 **/
@Service
@Slf4j
public class MesCraftServiceImpl extends BaseMesService<MesCraft> implements IMesCraftService {

    @Autowired
    private IMesCraftDao mesCraftDao;

    @Override
    protected void setPackQueryBean(MesCraft bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getCraftCode(), "craftCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCraftName(), "craftName", packBean);
    }

    @Override
    protected void onInsertBean(MesCraft item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCraftCode(), "工艺代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCraftCode(), "craftCode", roadPackBean);

        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("工艺代码【%s】已经存在，请检查数据", item.getCraftCode())
                    .build();
        }

    }

    @Override
    protected void onUpdateBean(MesCraft item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCraftCode(), "工艺代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCraftCode(), "craftCode", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("工艺代码【%s】已经存在，请检查数据", item.getCraftCode())
                    .build();
        }
    }

    @Override
    public ListPager<MesCraft> findMesCraft(MesCraft mesCraft, Pager pager) {
        return mesCraftDao.findMesCraft(mesCraft, pager);
    }
}
