package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCustomerDockService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerDock;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class MesCustomerDockService extends BaseMesService<MesCustomerDock> implements IMesCustomerDockService {


    protected void onInsertBean(MesCustomerDock item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCustCode(), "客户代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustCode(), "custCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getCustDockCode(), "custDockCode", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】客户代码【%s】道口代码已经存在，请检查数据", item.getCustCode(), item.getCustDockCode())
                    .build();
        }

    }

    protected void onUpdateBean(MesCustomerDock item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCustCode(), "客户代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustCode(), "custCode", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getCustDockCode(), "custDockCode", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】客户代码【%s】道口代码已经存在，请检查数据", item.getCustCode(), item.getCustDockCode())
                    .build();
        }
    }

    protected void setPackQueryBean(MesCustomerDock bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getCustCode(), "custCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustDockCode(), "custDockCode", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getCustDockType(), "custDockType", packBean);
    }

    @Override
    public void validateImport(List<MesCustomerDock> beanList) {

        for (MesCustomerDock bean : beanList) {

            // 数据校验
            ValidatorBean.checkNotNull(bean.getCustCode(), "客户代码不能为空");
            ValidatorBean.checkNotNull(bean.getCustDockCode(), "客户道口代码不能为空");
            ValidatorBean.checkNotNull(bean.getCustDockDesc(), "客户道口描述不能为空");

            //todo
            //查询数据库校验字段，待确定
//            DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
//            DdlPreparedPack.getStringEqualPack(bean.getCustCode(), "custCode", packBean);
//            MesCustomer customer = customerRepository.getByProperty(packBean);
//            if (StringUtils.isEmpty(customer)) {
//                throw new ImppBusiException(String.format("客户代码【%s】不存在", bean.getCustCode()));
//            }
//
//            bean.setCustName(customer.getCustName());
        }
    }

}
