package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesCustomerPartService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCarSeriesLoosePart;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerCarModelDetail;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerPart;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerCarModelDetailRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description:
 * @CreateDate: 2023/10/11 15:38
 * @Author: simon.song
 */
@Slf4j
@Service
public class MesCustomerPartService extends BaseMesService<MesCustomerPart> implements IMesCustomerPartService {


    @Autowired
    private MesCustomerCarModelDetailRepository mesCustomerCarModelDetailRDao;

    protected void setPackQueryBean(MesCustomerPart bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getCustPartNo(), "custPartNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getErpPartNo(), "erpPartNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getErpPartName(), "erpPartName", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustCode(), "custCode", packBean);
    }

    protected void onInsertBean(MesCustomerPart item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCustCode(), "客户代码不能为空");
        ValidatorBean.checkNotNull(item.getErpPartNo(), "零件号不能为空");
        ValidatorBean.checkNotNull(item.getCustPartNo(), "客户零件号不能为空");

        DdlPackBean cellBinPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustCode(), "custCode", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getErpPartNo(), "erpPartNo", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getCustPartNo(), "custPartNo", cellBinPackBean);
        boolean flg = baseRDao.isExitByHql(cellBinPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】客户零件号【%s】零件号【%s】客户代码已经存在，请检查数据", item.getCustPartNo(), item.getErpPartNo(), item.getCustCode())
                    .build();
        }

    }

    protected void onUpdateBean(MesCarSeriesLoosePart item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getCustCode(), "客户代码不能为空");
        ValidatorBean.checkNotNull(item.getPartNo(), "零件号不能为空");
        ValidatorBean.checkNotNull(item.getCustPartNo(), "客户零件号不能为空");

        DdlPackBean cellBinPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getCustCode(), "custCode", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getCustPartNo(), "custPartNo", cellBinPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", cellBinPackBean);
        boolean flg = baseRDao.isExitByHql(cellBinPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】客户零件号【%s】零件号【%s】客户代码已经存在，请检查数据", item.getCustPartNo(), item.getPartNo(), item.getCustCode())
                    .build();
        }

    }


    @Override
    public List<MesCustomerPart> findMesCustomerPart(MesCustomerCarModelDetail mesPartProdGroupDetail) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesPartProdGroupDetail.getOrganizeCode());
        DdlPreparedPack.getStringLikerPack(mesPartProdGroupDetail.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesPartProdGroupDetail.getCarModelCode(), "carModelCode", ddlPackBean);
        List<MesCustomerCarModelDetail> mesCustomerCarModelDetails = mesCustomerCarModelDetailRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesCustomerCarModelDetails)) {
            return new ArrayList<>();
        }
        List<String> customerPartNoList = mesCustomerCarModelDetails.stream().map(MesCustomerCarModelDetail::getCustPartNo).collect(Collectors.toList());
        ddlPackBean = DdlPackBean.getDdlPackBean(mesPartProdGroupDetail.getOrganizeCode());
        DdlPreparedPack.getInPackList(customerPartNoList, "custPartNo", ddlPackBean);
        return baseRDao.findByHqlWhere(ddlPackBean);
    }

}
