package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesDefectTypeService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesDefectType;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class MesDefectTypeService extends BaseMesService<MesDefectType> implements IMesDefectTypeService {

    protected void setPackQueryBean(MesDefectType bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getDefectTypeCode(), "defectTypeCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getDefectTypeName(), "defectTypeName", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getDefectType(), "defectType", packBean);
    }

    protected void onInsertBean(MesDefectType item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getDefectTypeCode(), "缺陷类型代码不能为空");
        ValidatorBean.checkNotNull(item.getDefectTypeName(), "缺陷类型名称不能为空");

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getDefectTypeCode(), "defectTypeCode", packBean);
        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】缺陷类型代码已经存在，请检查数据", item.getDefectTypeCode())
                    .build();
        }
    }


    protected void onUpdateBean(MesDefectType item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getDefectTypeCode(), "缺陷类型代码不能为空");
        ValidatorBean.checkNotNull(item.getDefectTypeName(), "缺陷类型名称不能为空");

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getDefectTypeCode(), "defectTypeCode", packBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", packBean);
        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】缺陷类型代码已经存在，请检查数据", item.getDefectTypeCode())
                    .build();
        }
    }

}
