package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesEnumDetailService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesEnumDetail;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


/**
 * @Description: 枚举明细配置
 * @Author: gsz
 * @Date: 2024/5/21 14:16
 * @Modify:
 */
@Service
@Slf4j
public class MesEnumDetailServiceImpl extends BaseMesService<MesEnumDetail> implements IMesEnumDetailService {

    @Override
    protected void setPackQueryBean(MesEnumDetail bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getEnumCode(), "enumCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getDetailCode(), "detailCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getDetailName(), "detailName", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getDetailValue(), "detailValue", packBean);
    }
    @Override
    protected void onInsertBean(MesEnumDetail item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getOrganizeCode(), "工厂号不能为空");
        ValidatorBean.checkNotNull(item.getEnumCode(), "enumCode不能为空" );
        ValidatorBean.checkNotNull(item.getDetailCode(), "detailCode不能为空");
        ValidatorBean.checkNotNull(item.getDetailName(), "detailName不能为空");
        ValidatorBean.checkNotNull(item.getDetailValue(), "detailValue不能为空");
        ValidatorBean.checkNotNull(item.getSeq(), "seq不能为空");

        //唯一性校验
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getEnumCode(), "enumCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getDetailCode(), "detailCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getDetailValue(), "detailValue", packBean);

        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("枚举代码【%s】属性代码【%s】属性值【%s】已经存在，请检查数据", item.getEnumCode(),item.getDetailCode(),item.getDetailValue())
                    .build();
        }
    }
}
