package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesEquitmentService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipment;
import cn.estsh.i3plus.pojo.mes.repository.MesWcEquipmentRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesEquipmentService extends BaseMesService<MesEquipment> implements IMesEquitmentService {

    @Autowired
    private MesWcEquipmentRepository mesWcEquipmentRepository;

    @Override
    protected void setPackQueryBean(MesEquipment bean, DdlPackBean packBean) {
        if (!StringUtil.isEmpty(bean.getFilterWorkCell())) {
            //过滤已绑定工位
            DdlPackBean packBean1 = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getIsNotNull("equipmentCode", packBean1);
            DdlPreparedPack.getIsNotNull( "workCellCode", packBean1);

            List<MesWcEquipment> workCells = mesWcEquipmentRepository.findByHqlWhere(packBean1);
            DdlPreparedPack.getNotInPack(workCells.stream().map(MesWcEquipment::getEquipmentCode).collect(Collectors.toList()), "equipmentCode", packBean);
        }
        DdlPreparedPack.getStringEqualPack(bean.getEquipmentCode(), "equipmentCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getEquipmentName(), "equipmentName", packBean);
    }


    @Override
    public MesEquipment insert(MesEquipment bean) {
        //插入前构造并校验数据
        onInsertBean(bean);
        Integer equipId = 1;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean();
        packBean.setOrderByStr(" order by equipId desc");
        MesEquipment mesEquipment = baseRDao.getByProperty(packBean);
        if (mesEquipment != null && mesEquipment.getEquipId() != null) {
            equipId = mesEquipment.getEquipId() + 1;
        }
        bean.setEquipId(equipId);
        bean = baseRDao.insert(bean);
        return bean;
    }

    @Override
    public MesEquipment update(MesEquipment bean) {
        //插入前构造并校验数据
        onUpdateBean(bean);

        baseRDao.update(bean);
        return bean;
    }


    protected void onInsertBean(MesEquipment item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备代码不能为空");


        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", seriesPackBean);
        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】设备代码已经存在，请检查数据", item.getEquipmentCode())
                    .build();
        }

    }

    protected void onUpdateBean(MesEquipment item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备代码不能为空");


        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", seriesPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", seriesPackBean);

        boolean flg = baseRDao.isExitByHql(seriesPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】设备代码已经存在，请检查数据", item.getEquipmentCode())
                    .build();
        }
    }
}
