package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesEquipmentSpotCheckDetailService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCraftRoute;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentSpotCheckDetail;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class MesEquipmentSpotCheckDetailService extends BaseMesService<MesEquipmentSpotCheckDetail> implements IMesEquipmentSpotCheckDetailService {

    protected void setPackQueryBean(MesEquipmentSpotCheckDetail bean, DdlPackBean packBean) {
        DdlPreparedPack.getNumEqualPack(bean.getPid(), "pid", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getSpotCheckItemCode(), "spotCheckItemCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getSpotCheckItemName(), "spotCheckItemName", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getDataType(), "dataType", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getSpotCheckItemType(), "spotCheckItemType", packBean);
    }


    @Override
    protected void onInsertBean(MesEquipmentSpotCheckDetail item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getSpotCheckItemCode(), "点检项代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(item.getPid(), "pid", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getSpotCheckItemCode(), "spotCheckItemCode", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("点检项代码【%s】已经存在，请检查数据", item.getSpotCheckItemCode())
                    .build();
        }
    }

    @Override
    protected void onUpdateBean(MesEquipmentSpotCheckDetail item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getSpotCheckItemCode(), "点检项代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(item.getPid(), "pid", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getSpotCheckItemCode(), "spotCheckItemCode", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("点检项代码【%s】已经存在，请检查数据", item.getSpotCheckItemCode())
                    .build();
        }
    }


}
