package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesLoadingGroupDetailService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesLoadingGroupDetail;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @Description : 装车组详情
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/5/14 15:13
 * @Modify:
 **/
@Service
@Slf4j
public class MesLoadingGroupDetailServiceImpl extends BaseMesService<MesLoadingGroupDetail> implements IMesLoadingGroupDetailService {

    @Override
    protected void setPackQueryBean(MesLoadingGroupDetail bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getLoadingGroupNo(), "loadingGroupNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getShippingGroupCode(), "shippingGroupCode", packBean);
    }


    @Override
    protected void onInsertBean(MesLoadingGroupDetail item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getShippingGroupCode(), "发运组代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getLoadingGroupNo(), "loadingGroupNo", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getShippingGroupCode(), "shippingGroupCode", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("发运组代码【%s】已经存在，请检查数据", item.getShippingGroupCode())
                    .build();
        }
    }

    @Override
    protected void onUpdateBean(MesLoadingGroupDetail item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getShippingGroupCode(), "发运组代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getLoadingGroupNo(), "loadingGroupNo", roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getShippingGroupCode(), "shippingGroupCode", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("发运组代码【%s】已经存在，请检查数据", item.getShippingGroupCode())
                    .build();
        }
    }

}