package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesLoadingGroupService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesLoadingGroup;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @Description : 装车组
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/5/14 15:13
 * @Modify:
 **/
@Service
@Slf4j
public class MesLoadingGroupServiceImpl extends BaseMesService<MesLoadingGroup> implements IMesLoadingGroupService {

    @Override
    protected void setPackQueryBean(MesLoadingGroup bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getLoadingGroupNo(), "loadingGroupNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getLoadingGroupName(), "loadingGroupName", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustCode(), "custCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustOrganizeCode(), "custOrganizeCode", packBean);
    }

    @Override
    protected void onInsertBean(MesLoadingGroup item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCustCode(), "客户代码不能为空");
        ValidatorBean.checkNotNull(item.getCustOrganizeCode(), "客户工厂代码不能为空");
        ValidatorBean.checkNotNull(item.getLoadingGroupNo(), "装车组代码不能为空");
        ValidatorBean.checkNotNull(item.getLoadingGroupName(), "装车组名称不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getLoadingGroupNo(), "loadingGroupNo", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("装车组代码【%s】已经存在，请检查数据", item.getLoadingGroupNo())
                    .build();
        }
    }

    @Override
    protected void onUpdateBean(MesLoadingGroup item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getCustCode(), "客户代码不能为空");
        ValidatorBean.checkNotNull(item.getCustOrganizeCode(), "客户工厂代码不能为空");
        ValidatorBean.checkNotNull(item.getLoadingGroupNo(), "装车组代码不能为空");
        ValidatorBean.checkNotNull(item.getLoadingGroupName(), "装车组名称不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getLoadingGroupNo(), "loadingGroupNo", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("装车组代码【%s】已经存在，请检查数据", item.getLoadingGroupNo())
                    .build();
        }
    }

}
