package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesLoadingListService;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.mes.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPartShippingGroup;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingList;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingListDetail;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagement;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.MesLoadingListDetailRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartShippingGroupRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesShippingOrderManagementRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class MesLoadingListService extends BaseMesService<MesLoadingList> implements IMesLoadingListService {


    @Autowired
    private MesLoadingListDetailService detailService;

    @Autowired
    private MesPartShippingGroupRepository partShippingGroupRepository;
    @Autowired
    private MesShippingOrderManagementRepository shippingOrderManagementRepository;
    @Autowired
    private MesLoadingListDetailRepository listDetailRepository;

    @Autowired
    private ISyncFuncService syncFuncService;


    @Override
    public MesLoadingList insert(MesLoadingList bean) {
        //插入前构造并校验数据
        onInsertBean(bean);

        MesShippingOrderManagement shippingOrderManagement = bean.getShippingOrderNoList().get(0);
        MesPartShippingGroup partShippingGroup = partShippingGroupRepository.getById(shippingOrderManagement.getPartShippingGroupId());

        if (StringUtils.isEmpty(partShippingGroup)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组数据为空，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }

        if (StringUtils.isEmpty(partShippingGroup.getIsLoading()) || partShippingGroup.getIsLoading() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否有装车不能为空，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }
        if (partShippingGroup.getIsLoading() == MesCommonConstant.FALSE_INTEGER) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否有装车配置为否，则不需要生成装车单，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }

        if (StringUtils.isEmpty(partShippingGroup.getIsFirstInstall()) || partShippingGroup.getIsFirstInstall() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否先装车不能为空，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }
        if (StringUtils.isEmpty(partShippingGroup.getIsEnableMemory()) || partShippingGroup.getIsEnableMemory() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否启用记忆不能为空，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }

        //是否逆转
        if (partShippingGroup.getIsReverse() == MesCommonConstant.TRUE_INTEGER && partShippingGroup.getReverseQuantity() < 1) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组配置是否逆转为【是】，逆转数不能小于1，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }


        //生成工单号
        GenSerialNoModel genSerialNoModel = new GenSerialNoModel(MesCommonConstant.LOADING_LIST_CODE);
//        genSerialNoModel.setPartNo(bean.getPartNo());
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();

        bean.setPartShippingGroupCode(partShippingGroup.getShippingGroupCode());
        bean.setShippingGroupName(partShippingGroup.getShippingGroupName());
        bean.setStatus(MesExtEnumUtil.LOADING_DETAIL_ORDER_STATUS.CREATE.getValue());
        bean.setOrderCode(resultList.get(0));
        bean.setIsFirstInstall(partShippingGroup.getIsFirstInstall());
        bean.setIsEnableMemory(partShippingGroup.getIsEnableMemory());
//        bean.setSeq(bean.getOrderCode());
        bean.setStatus(MesExtEnumUtil.LOADING_ORDER_STATUS.CREATE.getValue());
        String customerCode = bean.getShippingOrderNoList().get(0).getCustCode();
        String customerName = bean.getShippingOrderNoList().get(0).getCustName();
        String customerOrganizeCode = bean.getShippingOrderNoList().get(0).getCustOrganizeCode();
        String customerOrganizeName = bean.getShippingOrderNoList().get(0).getCustOrganizeName();
        bean.setCustCode(customerCode);
        bean.setCustName(customerName);
        bean.setCustOrganizeCode(customerOrganizeCode);
        bean.setCustOrganizeName(customerOrganizeName);
        bean = baseRDao.insert(bean);

        List<MesLoadingListDetail> detailList = new ArrayList<>();
        MesLoadingList finalBean = bean;

        bean.getShippingOrderNoList().forEach(k -> {
            MesLoadingListDetail detail = new MesLoadingListDetail();

            k.setOrderCode(finalBean.getOrderCode());

            BeanUtils.copyProperties(k, detail, "id");
            detail.setStatus(MesExtEnumUtil.LOADING_ORDER_DETAIL_SHIPPING_STATUS.PUBLISHED.getValue());
            detail.setPid(finalBean.getId());
            detail.setShippingId(k.getId());
            ConvertBean.serviceModelInitialize(detail, AuthUtil.getSessionUser().getUserName());
            ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            detailList.add(detail);
        });

        shippingOrderManagementRepository.saveAll(bean.getShippingOrderNoList());
        detailService.insertBatch(detailList);

        return bean;
    }

    @Override
    public void deleteWeaklyByIds(Long[] ids, String userName) {

        for (Long id : ids) {
            MesLoadingList bean = baseRDao.getById(id);
            if (StringUtils.isEmpty(bean)) continue;
            DdlPackBean detailPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getNumEqualPack(bean.getId(), "pid", detailPackBean);
            List<MesLoadingListDetail> detailDeleteList = detailService.findAll(detailPackBean);
            detailDeleteList.forEach(k -> {
                k.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            });
            bean.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            ConvertBean.serviceModelUpdate(bean, AuthUtil.getSessionUser().getUserName());
            baseRDao.update(bean);
            detailService.updates(detailDeleteList);
        }
    }


    protected void onInsertBean(MesLoadingList item) {

        List<String> shippingOrderNoList = new ArrayList<>();
        List<String> listDetailByShippingCodeList = new ArrayList<>();
        // 数据校验
        ValidatorBean.checkNotNull(item.getTruckNo(), "车牌号不能为空");
        ValidatorBean.checkNotNull(item.getWeather(), "天气不能为空");
        ValidatorBean.checkNotNull(item.getShippingOrderNoList(), "发运单不能为空");
        item.getShippingOrderNoList().forEach(k -> {
            if (k.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue() && k.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()) {
                shippingOrderNoList.add(k.getShippingCode());
            }

            DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getNumEqualPack(k.getId(), "shippingId", partPackBean);
            boolean flg = listDetailRepository.isExitByHql(partPackBean);
            if (flg) {
                listDetailByShippingCodeList.add(k.getShippingCode());
            }
        });

        if (!CollectionUtils.isEmpty(shippingOrderNoList)) {
            throw ImppExceptionBuilder.newInstance()
                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                    .setErrorDetail(String.format("发运单【%s】状态不是【%s】或【%s】,请重新选择数据", shippingOrderNoList, MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue(),
                            MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()))
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .build();
        }
        if (!CollectionUtils.isEmpty(listDetailByShippingCodeList)) {
            throw ImppExceptionBuilder.newInstance()
                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                    .setErrorDetail(String.format("发运单【%s】生成装车单,请重新选择数据", listDetailByShippingCodeList))
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .build();
        }

//        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
//        DdlPreparedPack.getStringEqualPack(item.getTruckNo(), "truckNo", partPackBean);
//        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.LOADING_ORDER_STATUS.CREATE.getValue(), "status", partPackBean);
//        boolean flg = baseRDao.isExitByHql(partPackBean);
//        if (flg) {
//            throw ImppExceptionBuilder.newInstance()
//                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
//                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
//                    .setErrorDetail("车牌号【%s】已经生成创建状态的装车单，请检查数据", item.getTruckNo())
//                    .build();
//        }

    }

    protected void setPackQueryBean(MesLoadingList bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getOrderCode(), "orderCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustCode(), "custCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustOrganizeCode(), "custOrganizeCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getTruckNo(), "truckNo", packBean);
    }
}
