package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesMouldMappingCfgService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;



/**
 * @Description : 模具映射关系维护
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/5/20 10:06
 * @Modify:
 **/
@Service
@Slf4j
public class MesMouldMappingCfgServiceImpl extends BaseMesService<MesMouldMappingCfg> implements IMesMouldMappingCfgService {

    @Autowired
    private MesEquipmentRepository mesEquipmentRDao;

    @Override
    protected void setPackQueryBean(MesMouldMappingCfg bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getAssetNum(), "assetNum", packBean);
//        DdlPreparedPack.getStringEqualPack(bean.getEquipmentCode(), "equipmentCode", packBean);
    }
    @Override
    protected void onInsertBean(MesMouldMappingCfg item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getOrganizeCode(), "工厂号不能为空");
        ValidatorBean.checkNotNull(item.getAssetNum(), "assetNum不能为空" );
        ValidatorBean.checkNotNull(item.getMouldNo(), "mouldNo不能为空");
        ValidatorBean.checkNotNull(item.getEquipmentCode(), "equipmentCode不能为空");

        MesEquipment mesEquipmentDb = mesEquipmentRDao.getByProperty(
                new String[]{MesConstWords.ORGANIZE_CODE, MesConstWords.IS_DELETED, MesConstWords.IS_VALID, "equipmentCode"},
                new Object[]{item.getOrganizeCode(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),
                        CommonEnumUtil.IS_VAILD.VAILD.getValue(), item.getEquipmentCode()});
        if (null == mesEquipmentDb) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("设备代码【%s】无效", item.getEquipmentCode())
                    .build();
        }
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getAssetNum(), "assetNum", packBean);
        DdlPreparedPack.getStringEqualPack(item.getMouldNo(), "mouldNo", packBean);
        DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", packBean);

        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("maximo模具号【%s】设备代码【%s】已经存在，请检查数据", item.getAssetNum(),item.getEquipmentCode())
                    .build();
        }
    }
}
