package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesShiftService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesShift;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalTime;
import java.time.temporal.ChronoUnit;
import java.util.List;

@Service
@Slf4j
public class MesShiftService extends BaseMesService<MesShift> implements IMesShiftService {

    protected void setPackQueryBean(MesShift bean, DdlPackBean packBean) {

        DdlPreparedPack.getNumEqualPack(bean.getIsValid(), "isValid", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getShiftCode(), "shiftCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getShiftName(), "shiftName", packBean);

    }

    protected void onInsertBean(MesShift item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getAreaCode(), "区域不能为空");
        ValidatorBean.checkNotNull(item.getShiftCode(), "班次代码不能为空");
        ValidatorBean.checkNotNull(item.getShiftName(), "班次名称不能为空");
        ValidatorBean.checkNotNull(item.getStartTime(), "开始时间不能为空");
        ValidatorBean.checkNotNull(item.getEndTime(), "结束时间不能为空");
        ValidatorBean.checkNotNull(item.getBeginDate(), "执行开始日期不能为空");
        ValidatorBean.checkNotNull(item.getEndDate(), "执行结束日期不能为空");
/*
        ValidatorBean.checkNotNull(item.getWorkerQty(), "作业人数不能为空");
*/

        DdlPackBean cellBinPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getAreaCode(), "areaCode", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getShiftCode(), "shiftCode", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", cellBinPackBean);
        cellBinPackBean.setWhereAppend(cellBinPackBean.getWhereAppend() + " " +
                " and ((model.beginDate <= '" + item.getBeginDate() + "' and model.endDate >= '" + item.getBeginDate() + "')" +
                " or (model.beginDate <= '" + item.getEndDate() + "' and model.endDate >= '" + item.getEndDate() + "')" +
                " or (model.beginDate >= '" + item.getBeginDate() + "' and model.endDate <= '" + item.getEndDate() + "'))");
        boolean flg = baseRDao.isExitByHql(cellBinPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】区域【%s】产线【%s】班次代码生效时间内已经存在有效数据，请检查数据", item.getAreaCode(), item.getWorkCenterCode(), item.getShiftCode())
                    .build();
        }
        //计算工时
        item.setWorkTimes(houresBetweenTwoTime(item.getStartTime().split(":"),item.getEndTime().split(":")));
    }



    protected void onUpdateBean(MesShift item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getAreaCode(), "区域不能为空");
        ValidatorBean.checkNotNull(item.getShiftCode(), "班次代码不能为空");
        ValidatorBean.checkNotNull(item.getShiftName(), "班次名称不能为空");
        ValidatorBean.checkNotNull(item.getStartTime(), "开始时间不能为空");
        ValidatorBean.checkNotNull(item.getEndTime(), "结束时间不能为空");
        ValidatorBean.checkNotNull(item.getBeginDate(), "执行开始日期不能为空");
        ValidatorBean.checkNotNull(item.getEndDate(), "执行结束日期不能为空");
/*
        ValidatorBean.checkNotNull(item.getWorkerQty(), "作业人数不能为空");
*/

        DdlPackBean cellBinPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getAreaCode(), "areaCode", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getShiftCode(), "shiftCode", cellBinPackBean);
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", cellBinPackBean);
        cellBinPackBean.setWhereAppend(cellBinPackBean.getWhereAppend() + " " +
                " and ((model.beginDate <= '" + item.getBeginDate() + "' and model.endDate >= '" + item.getBeginDate() + "')" +
                " or (model.beginDate <= '" + item.getEndDate() + "' and model.endDate >= '" + item.getEndDate() + "')" +
                " or (model.beginDate >= '" + item.getBeginDate() + "' and model.endDate <= '" + item.getEndDate() + "'))");
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", cellBinPackBean);
        boolean flg = baseRDao.isExitByHql(cellBinPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】区域【%s】产线【%s】班次代码生效时间内已经存在有效数据，请检查数据", item.getAreaCode(), item.getWorkCenterCode(), item.getShiftCode())
                    .build();
        }
        //计算工时
        item.setWorkTimes(houresBetweenTwoTime(item.getStartTime().split(":"),item.getEndTime().split(":")));
    }

    @Override
    public List<MesShift> findMesShift(MesShift mesShift) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesShift.getOrganizeCode());
        String nowDate = TimeTool.getNowTime(true);
        DdlPreparedPack.getNumEqualPack(mesShift.getIsValid(), "isValid", packBean);
        DdlPreparedPack.getStringEqualPack(mesShift.getShiftCode(), "shiftCode", packBean);
        DdlPreparedPack.getStringEqualPack(mesShift.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringLikerPack(mesShift.getShiftName(), "shiftName", packBean);
        DdlPreparedPack.getStringSmallerPack(nowDate, "beginDate", packBean);
        DdlPreparedPack.getStringBiggerPack(nowDate, "endDate", packBean);
        return baseRDao.findByHqlWhere(packBean);
    }


    public double houresBetweenTwoTime(String[] start, String[] end) {
        // 定义起始时间和结束时间
        LocalTime startTime = LocalTime.of(Integer.parseInt(start[0]), Integer.parseInt(start[1]), Integer.parseInt(start[2]));
        LocalTime endTime = LocalTime.of(Integer.parseInt(end[0]), Integer.parseInt(end[1]), Integer.parseInt(end[2]));

        // 计算时间间隔
        long hoursDifference = ChronoUnit.HOURS.between(startTime, endTime);

        // 由于时间跨越了午夜，我们需要手动计算实际的小时差
        if (endTime.isBefore(startTime)) {
            // 计算从开始时间到午夜的小时数
            long hoursToMidnight = ChronoUnit.HOURS.between(startTime, LocalTime.MAX);
            // 计算从午夜到结束时间的小时数
            long hoursFromMidnight = ChronoUnit.HOURS.between(LocalTime.MIN, endTime);
            // 将两部分小时数相加
            hoursDifference = hoursToMidnight + hoursFromMidnight + 1;
        }
        return (double) hoursDifference;
    }
}
