package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesWcEquipmentService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipment;
import cn.estsh.i3plus.pojo.mes.bean.MesWcEquipment;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * @Description :工作单元设备关系
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/6 15:52
 * @Modify:
 **/
@Service
@Slf4j
public class MesWcEquipmentServiceImpl extends BaseMesService<MesWcEquipment> implements IMesWcEquipmentService {

    @Autowired
    private MesEquipmentRepository mesEquipmentRDao;

    protected void setPackQueryBean(MesWcEquipment bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getAreaCode(), "areaCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCellCode(), "workCellCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getEquipmentCode(), "equipmentCode", packBean);
    }

    protected void onInsertBean(MesWcEquipment item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备代码不能为空");
        ValidatorBean.checkNotNull(item.getWorkCellCode(), "工作单元代码不能为空");
        ValidatorBean.checkNotNull(item.getWorkCenterCode(), "工作中心代码不能为空");

        //校验设备代码是否有效，校验设备是否已经绑定该工位，校验设备是否已经绑定其他生产线
        this.checkSaveOrUpdateMesWcEquipment(item);


    }

    protected void onUpdateBean(MesWcEquipment item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备代码不能为空");
        ValidatorBean.checkNotNull(item.getWorkCellCode(), "工作单元代码不能为空");
        ValidatorBean.checkNotNull(item.getWorkCenterCode(), "工作中心代码不能为空");
        //校验数据是否存在
        MesWcEquipment mesWcEquipmentDb = baseRDao.getById(item.getId());
        if (null == mesWcEquipmentDb ||
                (null != mesWcEquipmentDb && CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == mesWcEquipmentDb.getIsDeleted())) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("数据不存在")
                    .build();
        }
        //校验设备代码是否有效，校验设备是否已经绑定该工位，校验设备是否已经绑定其他生产线
        this.checkSaveOrUpdateMesWcEquipment(item);

    }

    /**
     * 新增/更新mes设备台账跟工位绑定-校验
     *
     * @param mesWcEquipment
     */
    private void checkSaveOrUpdateMesWcEquipment(MesWcEquipment mesWcEquipment) {
        //校验设备代码是否有效
        MesEquipment mesEquipmentDb = mesEquipmentRDao.getByProperty(
                new String[]{MesConstWords.ORGANIZE_CODE, MesConstWords.IS_DELETED, MesConstWords.IS_VALID, "equipmentCode"},
                new Object[]{mesWcEquipment.getOrganizeCode(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),
                        CommonEnumUtil.IS_VAILD.VAILD.getValue(), mesWcEquipment.getEquipmentCode()});
        if (null == mesEquipmentDb) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("设备代码【%s】无效", mesWcEquipment.getEquipmentCode())
                    .build();
        }

        //查询设备已关联数据
        List<MesWcEquipment> mesWcEquipments = baseRDao.findByProperty(
                new String[]{MesConstWords.ORGANIZE_CODE, MesConstWords.IS_DELETED, "equipmentCode"},
                new Object[]{mesWcEquipment.getOrganizeCode(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),
                        mesWcEquipment.getEquipmentCode()});
        if (!CollectionUtils.isEmpty(mesWcEquipments)) {
            for (MesWcEquipment wc : mesWcEquipments) {
                boolean isCheck = true;
                //更新时校验设备是否已经绑定该工位需要判断ID
                if (!StringUtils.isEmpty(mesWcEquipment.getId()) && mesWcEquipment.getId().equals(wc.getId())) {
                    isCheck = false;
                }
                if (isCheck) {
                    //校验设备是否已经绑定该工位
                    if (mesWcEquipment.getWorkCellCode().equals(wc.getWorkCellCode())) {
                        throw ImppExceptionBuilder.newInstance()
                                .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                                .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                                .setErrorDetail("工位代码【%s】与设备代码【%s】已经绑定，不能重复绑定"
                                        , mesWcEquipment.getWorkCellCode(), mesWcEquipment.getEquipmentCode())
                                .build();
                    }
                }
                //校验设备是否已经绑定其他生产线
                if (wc.getId() !=mesWcEquipment.getId()  &&  !mesWcEquipment.getWorkCenterCode().equals(wc.getWorkCenterCode())) {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                            .setErrorDetail("不能绑定！设备代码【%s】，已经绑定生产线代码【%s】", mesWcEquipment.getEquipmentCode(), wc.getWorkCenterCode())
                            .build();
                }
            }
        }
    }

}
