package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesWindowService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesWindow;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
/**
 * @Description : MES_看板界面
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/6 15:52
 * @Modify:
 **/
@Service
@Slf4j
public class MesWindowServiceImpl extends BaseMesService<MesWindow> implements IMesWindowService {

    @Override
    protected void setPackQueryBean(MesWindow bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getWindowNo(),"windowNo",packBean);
        DdlPreparedPack.getStringLikerPack(bean.getWindowName(),"windowName",packBean);
        DdlPreparedPack.getNumEqualPack(bean.getWindowType(),"windowType",packBean);
    }

    @Override
    protected void onInsertBean(MesWindow item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getWindowNo(), "界面编号不能为空");
        ValidatorBean.checkNotNull(item.getWindowName(), "界面名称不能为空");
        ValidatorBean.checkNotNull(item.getWindowType(), "界面类型不能为空");

        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWindowNo(), "windowNo", partPackBean);
        boolean flg = baseRDao.isExitByHql(partPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】界面编号已经存在，请检查数据", item.getWindowNo())
                    .build();
        }


    }

    @Override
    protected void onUpdateBean(MesWindow item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getWindowNo(), "界面编号不能为空");
        ValidatorBean.checkNotNull(item.getWindowName(), "界面名称不能为空");
        ValidatorBean.checkNotNull(item.getWindowType(), "界面类型不能为空");

        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWindowNo(), "windowNo", partPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", partPackBean);
        boolean flg = baseRDao.isExitByHql(partPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】界面编号已经存在，请检查数据", item.getWindowNo())
                    .build();
        }

    }

}
