package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.*;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesWorkOrderDao;
import cn.estsh.i3plus.ext.mes.apiservice.unit.exception.MesException;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.mes.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.mes.apiservice.util.DateUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.DateUtilExt;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.i3plus.pojo.model.wms.WmsProductModel;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesWorkOrderService extends BaseMesService<MesWorkOrder> implements IMesWorkOrderService {
    @Autowired
    private MesWorkOrderLogService workOrderLogService;
    @Autowired
    private ISyncFuncService syncFuncService;
    @Autowired
    private IMesPartService iMesPartService;
    @Autowired
    private MesProductVersionRepository mesProductVersionRDao;
    @Autowired
    private MesBomRepository mesBomRDao;
    @Autowired
    private IMesProductOffLineService mesProductOffLineService;
    @Autowired
    private IMesWorkOrderDao mesWorkOrderDao;
    @Autowired
    private MesWorkOrderRepository mesWorkOrderRDao;
    @Autowired
    private MesWorkCenterRepository mesWorkCenterRDao;
    @Autowired
    private MesShiftRepository mesShiftRDao;
    @Autowired
    private MesProductPlanRepository mesProductPlanRDao;
    @Autowired
    private IMesPartProdGroupService mesPartProdGroupService;
    @Autowired
    private MesCustomerPartRepository mesCustomerPartRDao;
    @Autowired
    private IMesPartSapService iMesPartSapService;
    @Autowired
    private IMesMoveRuleService mesMoveRuleService;
    @Autowired
    private MesProdRuleSortCfgRepository mesProdRuleSortCfgRDao;
    @Autowired
    private IMesProdRuleNosortCfgService mesProdRuleNosortCfgService;
    @Autowired
    private IMesPartSapService mesPartSapService;
    @Autowired
    private MesMoveRepository mesMoveRDao;

    @Override
    public MesWorkOrder insert(MesWorkOrder bean) {
        //插入前构造数据
        onInsertBean(bean);

        //获取相同类型 相同标识 生产序号最大值的
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderStatus(), "workOrderStatus", orderPackBean);
        DdlPreparedPack.getNumEqualPack(bean.getOrderFlag(), "orderFlag", orderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"produceSeq"}, orderPackBean);
        MesWorkOrder result = baseRDao.getByProperty(orderPackBean);


        GenSerialNoModel genSerialNoModel = new GenSerialNoModel();
        //若工单类型为排序 则根据排序插入工单数量插入多行 工单数量为1
        if (bean.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            Long produceSeq = 0L;
            if (null != result && result.getProduceSeq() != null) {
                produceSeq = (long) Math.toIntExact(result.getProduceSeq());
            }

            //生成工单号
            genSerialNoModel = new GenSerialNoModel(MesCommonConstant.MES_WORK_ORDER_NO);
            List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();

            MesWorkOrder copyMesWorkOrder = new MesWorkOrder();
            BeanUtils.copyProperties(bean, copyMesWorkOrder);
            copyMesWorkOrder.setProduceSeq((long) (produceSeq + 1));
            copyMesWorkOrder.setQty(1D);
            copyMesWorkOrder.setWorkOrderNo(resultList.get(0));
            ConvertBean.serviceModelInitialize(copyMesWorkOrder, bean.getCreateUser());

            MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
            BeanUtils.copyProperties(copyMesWorkOrder, workOrderLog, "id");

            baseRDao.insert(copyMesWorkOrder);
            workOrderLogService.insert(workOrderLog);
            //写入SAP下发计划表
            saveMesProductPlan(bean, true, false);
            return bean;
        } else if (bean.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()) {
            //生成工单号
            genSerialNoModel = new GenSerialNoModel(MesCommonConstant.MES_WORK_ORDER_NO);
        } else {
            //生成工单号
            genSerialNoModel = new GenSerialNoModel(MesCommonConstant.MES_WORK_ORDER_NO);
            Long produceSeq = 0L;
            if (null != result && result.getProduceSeq() != null) {
                produceSeq = (long) Math.toIntExact(result.getProduceSeq());
            }
            bean.setProduceSeq(produceSeq + 1);
            GenSerialNoModel orderSeqSerialNoModel = new GenSerialNoModel(MesCommonConstant.ORDER_NO_BTO_SEQ);
            List<String> resultList = syncFuncService.syncSerialNo(orderSeqSerialNoModel, bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();
            bean.setWorkOrderSeq(resultList.get(0));
//            bean.setOrderSeq(Integer.parseInt(resultList.get(0)));
        }
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();
        bean.setWorkOrderNo(resultList.get(0));
        bean.setQty(bean.getQty());

        ConvertBean.serviceModelInitialize(bean, bean.getCreateUser());

        MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
        BeanUtils.copyProperties(bean, workOrderLog, "id");
        workOrderLogService.insert(workOrderLog);
        bean = baseRDao.insert(bean);
        //写入SAP下发计划表
        saveMesProductPlan(bean, true, false);
        return bean;
    }

    private void saveMesProductPlan(MesWorkOrder bean, boolean isInsert, boolean isReport) {
//        if (StringUtil.isEmpty(bean.getPlanOrderNo())) {
//            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
//            DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "planPartNo", ddlPackBean);
//            DdlPreparedPack.getStringEqualPack(bean.getPlanStartTime(), "planStartDate", ddlPackBean);
//            DdlPreparedPack.getStringEqualPack(bean.getPlanEndTime(), "planEndDate", ddlPackBean);
//            MesProductPlan mesProductPlan = mesProductPlanRDao.getByProperty(ddlPackBean);
//            if (null == mesProductPlan) {
//                MesProductPlan mesPlanOrder = new MesProductPlan();
//                mesPlanOrder.setPlanOrderNo("");
//                mesPlanOrder.setPlanQty(bean.getQty());
//                mesPlanOrder.setPlanPartNo(bean.getPartNo());
//                mesPlanOrder.setCompleteQty(0d);
//                mesPlanOrder.setUncompleteQty(bean.getQty());
//                mesPlanOrder.setUnit(bean.getUnit());
//                mesPlanOrder.setPlanStartDate(bean.getPlanStartTime());
//                mesPlanOrder.setPlanEndDate(bean.getPlanEndTime());
//                mesPlanOrder.setPartMappingWorkCenterCode(bean.getErpWorkCenter());
//                mesPlanOrder.setPlanOrganizeCode(bean.getOrganizeCode());
//                mesPlanOrder.setOrganizeCode(bean.getOrganizeCode());
//                ConvertBean.serviceModelInitialize(mesPlanOrder, bean.getCreateUser());
//                mesProductPlanRDao.insert(mesPlanOrder);
//            } else {
//                if (isInsert) {
//                    mesProductPlan.setPlanQty(MathOperation.add(mesProductPlan.getPlanQty(), bean.getQty()));
//                } else {
//                    //报工
//                    if (isReport) {
//                        mesProductPlan.setCompleteQty(MathOperation.add(mesProductPlan.getCompleteQty(), bean.getNum()));
//                        //报工调整
//                    } else {
//                        mesProductPlan.setCompleteQty(MathOperation.sub(mesProductPlan.getCompleteQty(), bean.getNum()));
//                    }
//                }
//                mesProductPlan.setUncompleteQty(MathOperation.sub(mesProductPlan.getPlanQty(), mesProductPlan.getCompleteQty()));
//                ConvertBean.serviceModelUpdate(mesProductPlan, bean.getCreateUser());
//                mesProductPlanRDao.update(mesProductPlan);
//            }
//        } else {
//            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
//            DdlPreparedPack.getStringEqualPack(bean.getPlanOrderNo(), "planOrderNo", ddlPackBean);
//            DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "planPartNo", ddlPackBean);
//            MesProductPlan mesProductPlan = mesProductPlanRDao.getByProperty(ddlPackBean);
//            if (null == mesProductPlan) {
//                MesException.throwMesBusiException("SAP计划单号【%s】物料【%s】,不存在", bean.getPlanOrderNo(), bean.getPartNo());
//            }
//            if (!isInsert) {
//                if (Objects.isNull(mesProductPlan.getCompleteQty())) {
//                    mesProductPlan.setCompleteQty(0d);
//                }
//                //报工
//                if (isReport) {
//                    mesProductPlan.setCompleteQty(MathOperation.add(mesProductPlan.getCompleteQty(), bean.getNum()));
//                    //报工调整
//                } else {
//                    mesProductPlan.setCompleteQty(MathOperation.sub(mesProductPlan.getCompleteQty(), bean.getNum()));
//                }
//                mesProductPlan.setUncompleteQty(MathOperation.sub(mesProductPlan.getPlanQty(), mesProductPlan.getCompleteQty()));
//                ConvertBean.serviceModelUpdate(mesProductPlan, bean.getCreateUser());
//                mesProductPlanRDao.update(mesProductPlan);
//            }
//        }
    }

    @Override
    public MesWorkOrder update(MesWorkOrder bean) {
        MesWorkOrder result = baseRDao.getById(bean.getId());
        //若工单类型为排序 则根据排序插入工单数量插入多行 工单数量为1
        if (result.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            if (!Objects.equals(bean.getQty(), result.getQty())) {
                MesException.throwMesBusiException(String.format("工单类型为【%s】 不需要修改数量",
                        MesExtEnumUtil.ORDER_TYPE.SORT.getDescription()));
            }
        } else {
            //工单数量只能比原来的大
            if (bean.getQty() < result.getQty()) {
                MesException.throwMesBusiException(String.format("工单数量不能小于工单源数量【%s】",
                        result.getQty()));
            }
        }
        //在加工的工单不能修改物料信息
        if (result.getWorkOrderStatus() >= MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue()) {
            if (!Objects.equals(bean.getPartNo(), result.getPartNo())) {
                MesException.throwMesBusiException(String.format("工单状态为【%s】不允许修改物料",
                        MesExtEnumUtil.ORDER_TYPE.valueOfDescription(result.getWorkOrderStatus())));
            }
        }

        result.setQty(bean.getQty());
        //更新已汇报
        if (bean.getReportedQty() >= 0) {
            result.setReportedQty(bean.getReportedQty());
        }
        if (bean.getUnCompleteQty() >= 0) {
            result.setUnCompleteQty(bean.getUnCompleteQty());
        }
        if (bean.getAdjustQty() > 0) {
            result.setAdjustQty(bean.getAdjustQty());
        }
        //更新状态
        if (bean.getWorkOrderStatus() > 0) {
            if (MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue() == bean.getWorkOrderStatus()) {
                if (MesExtEnumUtil.ORDER_STATUS.CREATE.getValue() != result.getWorkOrderStatus()) {
                    MesException.throwMesBusiException(String.format("工单状态为【%s】不为创建，不允许发布",
                            MesExtEnumUtil.ORDER_TYPE.valueOfDescription(result.getWorkOrderStatus())));
                }
            }
            result.setWorkOrderStatus(bean.getWorkOrderStatus());
        }
        ConvertBean.serviceModelInitialize(bean, AuthUtil.getSessionUser().getUserName());

        MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
        BeanUtils.copyProperties(result, workOrderLog, "id");
        workOrderLogService.insert(workOrderLog);

        baseRDao.update(result);
        return bean;
    }


    @Override
    protected void setPackQueryBean(MesWorkOrder bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringLikerPack(bean.getWorkOrderNo(), "workOrderNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartName(), "partNameRdd", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCreateUser(), "createUser", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustPartNo(), "custPartNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderStatus(), "workOrderStatus", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getWorkOrderType(), "workOrderType", packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getCreateDateTimeStart(), "createDatetime", packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getCreateDateTimeEnd(), "createDatetime", packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getStartTimeStart(), "planStartTime", packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getStartTimeEnd(), "planStartTime", packBean);
        DdlPreparedPack.getStringBiggerPack(bean.getEndTimeStart(), "planEndTime", packBean);
        DdlPreparedPack.getStringSmallerPack(bean.getEndTimeEnd(), "planEndTime", packBean);
        packBean.setOrderByStr(" order by createDatetime desc,workOrderSeq asc,workOrderNo asc ");
    }

    @Override
    protected void onInsertBean(MesWorkOrder item) {

        // 数据校验
        ValidatorBean.checkNotNull(item.getWorkOrderType(), "工单类型不能为空");
        ValidatorBean.checkNotNull(item.getWorkCenterCode(), "产线不能为空");

        ValidatorBean.checkNotNull(item.getShiftCode(), "班次不能为空");

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesWorkCenter)) {
            MesException.throwMesBusiException("产线【%s】不存在", item.getWorkCenterCode());
        }
        item.setErpWorkCenter(mesWorkCenter.getErpWorkCenter());
        //校验标识
        //若工单类型为排序
        if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.SORT.getValue()) {
            ValidatorBean.checkNotNull(item.getOrderFlag(), "标识不能为空");
            //若工单类型为排序 标识为 N E P S
            if (item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue()) {
                throw ImppExceptionBuilder.newInstance()
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail(String.format("工单标识为【%s】请选择 【%s】、【%s】、【%s】、【%s】 标识",
                                item.getOrderFlag(), MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.S.getDescription()
                                , MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getDescription(),
                                MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getDescription(),
                                MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getDescription()))
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .build();
            }
            if (StringUtil.isEmpty(item.getPlanStartTime()) || StringUtil.isEmpty(item.getPlanEndTime())) {
                packPlanTime(item);
            }
            //生产版本为空
            if (StringUtil.isEmpty(item.getProductVersion())) {
                MesProductVersion mesProductVersion = getMesProductVersion(item, mesWorkCenter);
                item.setProductVersion(mesProductVersion.getProductVersion());
            }

            if (!StringUtil.isEmpty(item.getVinCode())) {
                //校验vin号是否重复
                ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(item.getVinCode(), "vinCode", ddlPackBean);
                if (baseRDao.isExitByHql(ddlPackBean)) {
                    MesException.throwMesBusiException("vin号【%s】重复", item.getVinCode());
                }
            }

            //校验零件生产组信息
            if (StringUtil.isEmpty(item.getPartProdGroupCode())) {
                MesPartProdGroup mesPartProdGroup = getMesPartProdGroup(item);
                item.setPartProdGroupCode(mesPartProdGroup.getPartProdGroupCode());
                item.setPartProdGroupName(mesPartProdGroup.getPartProdGroupName());
            }

            //排序工单数量为1
            item.setQty(1d);
            item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.CREATE.getValue());
        } else if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()) {
            ValidatorBean.checkNotNull(item.getOrderFlag(), "标识不能为空");
            ValidatorBean.checkNotNull(item.getProductVersion(), "生产版本不能为空");
            if (item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getValue() &&
                    item.getOrderFlag() != MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue()) {
                throw ImppExceptionBuilder.newInstance()
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail(String.format("工单标识为【%s】请选择 【%s】、【%s】 标识",
                                item.getOrderFlag(), MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.E.getDescription(),
                                MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getDescription()))
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .build();
            }
            if (StringUtil.isEmpty(item.getPlanStartTime()) || StringUtil.isEmpty(item.getPlanEndTime())) {
                packPlanTime(item);
            }
            if (StringUtil.isEmpty(item.getCustPartNo())) {
                getCustomerPart(item);
            }
            //校验物料生产版本是否存在
            checkMesProductVersion(item, mesWorkCenter);
            item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue());
        } else {
            ValidatorBean.checkNotNull(item.getPlanStartTime(), "计划开始时间不能为空");
            ValidatorBean.checkNotNull(item.getPlanEndTime(), "计划结束时间不能为空");
            //生产版本为空
            if (StringUtil.isEmpty(item.getProductVersion())) {
                MesProductVersion mesProductVersion = getMesProductVersion(item, mesWorkCenter);
                item.setProductVersion(mesProductVersion.getProductVersion());
            }
            if (StringUtil.isEmpty(item.getCustPartNo())) {
                getCustomerPart(item);
            }
            //拼接
//            item.setPlanStartTime(item.getPlanStartTime() + " " + "00:00:00");
//            item.setPlanEndTime(item.getPlanEndTime() + " " + "23:59:59");
            item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue());
        }
        //物料名称不存在，从物料信息中获取
        MesPartSap mesPart = iMesPartSapService.getMesPartSapByPartNo(item.getPartNo(), item.getOrganizeCode());
        item.setPartName(mesPart.getPartName());
        item.setUnit(mesPart.getUnit());
        //生产时间新增取当天
        if (StringUtil.isEmpty(item.getProduceTime())) {
            item.setProduceTime(TimeTool.getToday());
        }

    }

    private MesPartProdGroup getMesPartProdGroup(MesWorkOrder item) {
        MesPartProdGroupDetail mesPartProdGroupDetail = new MesPartProdGroupDetail();
        mesPartProdGroupDetail.setWorkCenterCode(item.getWorkCenterCode());
        mesPartProdGroupDetail.setPartNo(item.getPartNo());
        mesPartProdGroupDetail.setOrganizeCode(item.getOrganizeCode());
        List<MesPartProdGroup> mesPartProdGroupList = mesPartProdGroupService.findMesPartProdGroup(mesPartProdGroupDetail);
        if (CollectionUtils.isEmpty(mesPartProdGroupList)) {
            MesException.throwMesBusiException("生产线代码【%s】零件号【%s】对应的零件生产组信息没有维护",
                    item.getWorkCenterCode(), item.getPartNo());
        }
        MesPartProdGroup mesPartProdGroup = mesPartProdGroupList.iterator().next();
        return mesPartProdGroup;
    }

    private void getCustomerPart(MesWorkOrder item) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "erpPartNo", ddlPackBean);
        MesCustomerPart customerPart = mesCustomerPartRDao.getByProperty(ddlPackBean);
        if (!Objects.isNull(customerPart)) {
            item.setCustPartNo(customerPart.getCustPartNo());
            item.setCustomerPartName(customerPart.getCustPartName());
        }
    }

    private void checkMesProductVersion(MesWorkOrder item, MesWorkCenter mesWorkCenter) {
        DdlPackBean ddlPackBean;
        ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkCenter.getErpWorkCenter(), "workCenterCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(item.getProductVersion(), "productVersion", ddlPackBean);
        if (!mesProductVersionRDao.isExitByHql(ddlPackBean)) {
            MesException.throwMesBusiException("物料【%s】产线【%s】生产版本【%s】信息不存在", item.getPartNo(), mesWorkCenter.getWorkCenterCode(), item.getProductVersion());
        }
    }

    private MesProductVersion getMesProductVersion(MesWorkOrder item, MesWorkCenter mesWorkCenter) {
        DdlPackBean ddlPackBean;
        ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkCenter.getErpWorkCenter(), "workCenterCode", ddlPackBean);
        ddlPackBean.setOrderByStr(" order by createDatetime desc ");
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesProductVersion)) {
            MesException.throwMesBusiException("物料【%s】工作中心【%s】对应的生产版本信息不存在", item.getPartNo(),
                    mesWorkCenter.getErpWorkCenter());
        }
        return mesProductVersion;
    }

    private void packPlanTime(MesWorkOrder item) {
        DdlPackBean ddlPackBean;
        ValidatorBean.checkNotNull(item.getPlanDate(), "计划日期不能为空");
        ValidatorBean.checkNotNull(item.getShiftCode(), "班次不能为空");
        //工单导入：计划时间不能为当天之前日期
        try {
            if (DateUtilExt.compareDateTime(DateUtil.SHORT_FORMAT, item.getPlanDate(), TimeTool.getToday()) < 0) {
                MesException.throwMesBusiException("导入工单计划日期不能小于今天");
            }
        } catch (ImppBusiException busExcep) {
            MesException.throwMesBusiException(busExcep.getErrorDetail());
        } catch (Exception e) {
            MesException.throwMesBusiException("计划日期格式异常");
        }
        //查询班次信息
        ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(item.getShiftCode(), "shiftCode", ddlPackBean);
        MesShift mesShift = mesShiftRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesShift)) {
            MesException.throwMesBusiException("产线【%s】班次【%s】信息不存在", item.getWorkCenterCode(), item.getShiftCode());
        }
        item.setShiftName(mesShift.getShiftName());
        //拼接
        item.setPlanStartTime(item.getPlanDate() + " " + mesShift.getStartTime());
        item.setPlanEndTime(item.getPlanDate() + " " + mesShift.getEndTime());
        try {
            int windowStartEndTimeCompare = DateUtilExt.compareDateTime(DateUtil.BASE_FORMAT, item.getPlanStartTime(), item.getPlanEndTime());
            if (windowStartEndTimeCompare > 0) {
                item.setPlanEndTime(TimeTool.pareDateToString(DateUtil.addDays(item.getPlanEndTime(), 1)));
            }
        } catch (Exception e) {
            MesException.throwMesBusiException("时间格式异常");
        }
        item.setProduceTime(item.getPlanDate());
    }

    @Override
    protected void onUpdateBean(MesWorkOrder item) {

    }

    @Override
    public void doProductReport(MesWorkOrder mesWorkOrder) {
        //查询物料信息
        MesPart mesPart = iMesPartService.getPartByPartNo(mesWorkOrder.getPartNo(), mesWorkOrder.getOrganizeCode());
        //查询产线信息
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesWorkCenter)) {
            MesException.throwMesBusiException("产线【%s】不存在", mesWorkOrder.getWorkCenterCode());
        }
        //获取生产版本
        MesProductVersion mesProductVersion = getMesProductVersion(mesWorkOrder);
        //物料+生产版本获取bom信息
        List<MesBom> mesBoms = getBomList(mesPart.getOrganizeCode(), mesPart, mesProductVersion.getAlternativePartList());
        //报工类型
        if (MesExtEnumUtil.REPORT_TYPE.REPORT.getValue() == mesWorkOrder.getReportType()) {
            mesWorkOrder.setReportedQty(MathOperation.add(mesWorkOrder.getNum(), mesWorkOrder.getReportedQty()));
            //更新SAP计划完成数量
            saveMesProductPlan(mesWorkOrder, false, true);
        } else {
            //冲销数量大于工单完成数量报错
            if (mesWorkOrder.getNum() > mesWorkOrder.getReportedQty()) {
                MesException.throwMesBusiException("冲销数量【%s】大于工单完成数量【%s】,不允许冲销",
                        mesWorkOrder.getNum(), mesWorkOrder.getReportedQty());
            }
            //报工调整数量加-
            mesWorkOrder.setAdjustQty((MathOperation.add(mesWorkOrder.getNum(), mesWorkOrder.getAdjustQty())));
            //已汇报数量减一
            mesWorkOrder.setReportedQty(MathOperation.sub(mesWorkOrder.getReportedQty(), mesWorkOrder.getNum()));
            //更新SAP计划完成数量
            saveMesProductPlan(mesWorkOrder, false, false);
        }
        //更新工单状态
        double unCompleteQty = MathOperation.sub(mesWorkOrder.getQty(), mesWorkOrder.getReportedQty());
        mesWorkOrder.setUnCompleteQty(unCompleteQty > 0 ? unCompleteQty : 0);
        if (mesWorkOrder.getReportedQty() > mesWorkOrder.getQty()) {
            MesException.throwMesBusiException("工单报工数量【%s】大于工单数量【%s】,不允许报工",
                    mesWorkOrder.getReportedQty(), mesWorkOrder.getQty());
        } else if (Objects.equals(mesWorkOrder.getReportedQty(), mesWorkOrder.getQty())) {
            mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
        } else {
            mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
        }
        update(mesWorkOrder);

        //生成条码
        List<String> resultList = getStringList(mesWorkOrder);
        //保存数据
        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
        for (String sn : resultList) {
            for (MesBom mesBom : mesBoms) {
                mesProductOffLineList.add(creatMesProductOffLine(mesWorkOrder, mesProductVersion, sn, mesBom, false));
            }
        }
        mesProductOffLineService.insertBatch(mesProductOffLineList);
    }


    public List<MesBom> getBomList(String organizeCode, MesPart mesPart, String bomVersion) {
        List<MesBom> bomData = new ArrayList<>();
        // 递归展开 BOM
        expendBomChild(organizeCode, mesPart.getPartNo(), bomVersion, bomData);

        if (bomData.size() == 0) {
            MesException.throwMesBusiException("零件号 %s 没找找到 ERP 同步的原始 BOM 数据，请检查接口是否正常！", mesPart.getPartNo());
        }

        Iterator<MesBom> iterator = bomData.iterator();
        while (iterator.hasNext()) {
            MesBom item = iterator.next();
            // 删除虚零件
            if (Objects.equals("X", item.getPartType())) {
                iterator.remove();
            } else {
                item.setPartNo(mesPart.getPartNo());
                item.setPartName(mesPart.getPartName());
            }
        }

        Map<String, MesBom> summaryBomMap = new HashMap<>();

        for (MesBom item : bomData) {
            String key = String.format("%s:%s", item.getPartNo().toUpperCase().trim(),
                    item.getItemPartNo().toUpperCase().trim());
            if (!summaryBomMap.containsKey(key)) {
                summaryBomMap.put(key, item);
            } else {
                MesBom tempItem = summaryBomMap.get(key);
                // 子零件重复的累加数量
                tempItem.setItemQty(MathOperation.add(tempItem.getItemQty(), item.getItemQty()));
            }
        }
        // 返回汇总数据
        return new ArrayList<>(summaryBomMap.values());
    }


    /**
     * 递归展开 BOM
     *
     * @param organizeCode 工厂代码
     * @param partNo       父零件号
     * @param bomData      存放 BOM 数据的集合
     */
    private void expendBomChild(String organizeCode, String partNo, String bomVersion, List<MesBom> bomData) {
        List<MesBom> bomDetails = getMesBoms(partNo, organizeCode, bomVersion);

        for (MesBom item : bomDetails) {
            MesBom newItem = new MesBom();
            BeanUtils.copyProperties(item, newItem);
            // 处理父零件用量不为 1 的情况
            if (newItem.getQty() != 1.0 && newItem.getQty() > 0.0) {
                Double rateQty = divide(newItem.getItemQty(), newItem.getQty(), 8);
                newItem.setQty(1.0);
                newItem.setItemQty(rateQty);
            }
            // 加入到 BOM 清单中
            bomData.add(newItem);
            // 如果有子阶，继续递归
            if (Objects.equals("X", item.getPartType())) {
                expendBomChild(organizeCode, newItem.getItemPartNo(), null, bomData);
            }
        }
    }

    @Override
    public void doProductReportExt(MesWorkOrder mesWorkOrderNew) {
        MesWorkOrder mesWorkOrder = baseRDao.getById(mesWorkOrderNew.getId());
        if (Objects.isNull(mesWorkOrder)) {
            MesException.throwMesBusiException("工单id为【%s】工单信息不存在", mesWorkOrderNew.getId());
        }
        //查询物料信息
        MesPart mesPart = iMesPartService.getPartByPartNo(mesWorkOrder.getPartNo(), mesWorkOrder.getOrganizeCode());
        if (mesWorkOrderNew.getNum() <= 0) {
            MesException.throwMesBusiException("工单报工数量不能为0,不允许报工");
        }
        //报工类型
        //报工
        if (MesExtEnumUtil.REPORT_TYPE.REPORT.getValue() == mesWorkOrder.getReportType()) {
            mesWorkOrder.setReportedQty(MathOperation.add(mesWorkOrderNew.getNum(), mesWorkOrder.getReportedQty()));
        } else {
            //冲销数量大于工单完成数量报错
            if (mesWorkOrderNew.getNum() > mesWorkOrder.getReportedQty()) {
                MesException.throwMesBusiException("冲销数量【%s】大于工单完成数量【%s】,不允许冲销",
                        mesWorkOrderNew.getNum(), mesWorkOrder.getReportedQty());
            }
            //报工调整数量加-
            mesWorkOrder.setAdjustQty((MathOperation.add(mesWorkOrderNew.getNum(), mesWorkOrder.getAdjustQty())));
            //已汇报数量减一
            mesWorkOrder.setReportedQty(MathOperation.sub(mesWorkOrder.getReportedQty(), mesWorkOrderNew.getNum()));
        }
        //更新工单状态
        double unCompleteQty = MathOperation.sub(mesWorkOrder.getQty(), mesWorkOrder.getReportedQty());
        mesWorkOrder.setUnCompleteQty(unCompleteQty > 0 ? unCompleteQty : 0);
        if (mesWorkOrder.getReportedQty() > mesWorkOrder.getQty()) {
            MesException.throwMesBusiException("工单报工数量大于工单数量【%s】,不允许报工", mesWorkOrder.getQty());
        } else if (Objects.equals(mesWorkOrder.getReportedQty(), mesWorkOrder.getQty())) {
            mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
        } else {
            mesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
        }
        //获取生产版本
        MesProductVersion mesProductVersion = getMesProductVersion(mesWorkOrder);
        //物料+生产版本获取bom信息
        List<MesBom> mesBoms = getBomList(mesPart.getOrganizeCode(), mesPart, mesProductVersion.getAlternativePartList());
        //生成条码
        List<String> resultList = getStringList(mesWorkOrderNew);
        //保存数据
        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
        List<MesMove> mesMoveList = new ArrayList<>();
        boolean isReport = false;
        boolean isMove = false;
        //排序工单
        if (MesExtEnumUtil.ORDER_TYPE.SORT.getValue() == mesWorkOrder.getWorkOrderType()) {
            //报工类型
            if (MesExtEnumUtil.REPORT_TYPE.WRITE_OFF.getValue() == mesWorkOrder.getReportType()) {
                MesException.throwMesBusiException("工单【%s】为排序工单不允许冲销", mesWorkOrder.getWorkOrderNo());
            }
            //排序加工规则
            MesProdRuleSortCfg mesProdRuleSortCfg = getMesProdRuleSortCfg(mesWorkOrder);
            if (MesExtEnumUtil.MES_REPORT_TYPE.REPORT.getValue() == mesProdRuleSortCfg.getReportType()) {
                isReport = true;
            } else if (MesExtEnumUtil.MES_REPORT_TYPE.MOVE.getValue() == mesProdRuleSortCfg.getReportType()) {
                isMove = true;
            } else if (MesExtEnumUtil.MES_REPORT_TYPE.REPORT_MOVE.getValue() == mesProdRuleSortCfg.getReportType()) {
                isReport = true;
                isMove = true;
            } else if (MesExtEnumUtil.MES_REPORT_TYPE.CUSTOMER_SUPPLY_MOVE.getValue() == mesProdRuleSortCfg.getReportType()) {
                customerSupplyMove(mesWorkOrder, mesProductVersion, mesBoms, resultList, mesProductOffLineList, mesMoveList);
            } else {
                log.info("工单{}排序加工规则报工类型未维护", mesWorkOrder.getWorkOrderNo());
                return;
            }

            if (MesExtEnumUtil.MES_REPORT_TYPE.CUSTOMER_SUPPLY_MOVE.getValue() != mesProdRuleSortCfg.getReportType()) {
                //查询SAP物料信息
                MesPartSap mesPartSap = mesPartSapService.getMesPartSapByPartNo(mesWorkOrder.getPartNo(), mesWorkOrder.getOrganizeCode());
                for (String sn : resultList) {
                    //成品汇报
                    if (isReport) {
                        for (MesBom mesBom : mesBoms) {
                            mesProductOffLineList.add(creatMesProductOffLine(mesWorkOrder, mesProductVersion, sn, mesBom, false));
                        }
                    }
                    //成品移库
                    if (isMove) {
                        MesMove move = createMove(mesPartSap, mesProdRuleSortCfg.getSrcErpLocation(), mesProdRuleSortCfg.getDestErpLocation(), mesWorkOrder.getOrganizeCode(), 1d, sn, MesExtEnumUtil.MOVE_TYPE.FINISH_PRODUCTS_MOVE.getValue());
                        move.setMatnr(mesWorkOrder.getPartNo());
                        mesMoveList.add(move);
                    }
                }
            }
        } else if (MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue() == mesWorkOrder.getWorkOrderType()) {
            MesProdRuleNosortCfg mesProdRuleNoSortCfg = mesProdRuleNosortCfgService.findMesProdRuleNosortCfgByWorkCenterCode(mesWorkOrder.getWorkCenterCode(), mesWorkOrder.getPartNo(), mesWorkOrder.getOrganizeCode());
            if (CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == mesProdRuleNoSortCfg.getReportType()) {
                for (String sn : resultList) {
                    for (MesBom mesBom : mesBoms) {
                        mesProductOffLineList.add(creatMesProductOffLine(mesWorkOrderNew, mesProductVersion, sn, mesBom, false));
                    }
                }
            }
        } else {
            log.info("工单{}排序规则不属于排序工单和非排序工单", mesWorkOrder.getWorkOrderNo());
            return;
        }

        //更新工单
        update(mesWorkOrder);
        //保存报工记录
        if (CollectionUtils.isNotEmpty(mesProductOffLineList)) {
            mesProductOffLineService.insertBatch(mesProductOffLineList);
        }
        //保存移库记录
        if (CollectionUtils.isNotEmpty(mesMoveList)) {
            mesMoveRDao.saveAll(mesMoveList);
        }
    }

    @Override
    public void deleteUnSortOrder(String userName, String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue(), "workOrderType", ddlPackBean);
        DdlPreparedPack.getStringSmallerNotEqualPack(TimeTool.getToday(), "planStartTime", ddlPackBean);
        DdlPreparedPack.getNumberSmallerPack(MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue(), "workOrderStatus", ddlPackBean);
        baseRDao.updateByProperties(new String[]{"modifyUser", "modifyDatetime", "workOrderStatus"},
                new Object[]{userName, TimeTool.getNowTime(true), MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue()}, ddlPackBean);
    }


    @Override
    public void doReportWorkReversal(WmsProductModel model) {
        //查询物料信息
        MesPart mesPart = iMesPartService.getPartByPartNo(model.getPartNo(), model.getOrganizeCode());
        //获取生产版本
        MesProductVersion mesProductVersion = getMesProductVersion(model);
        //获取bom
        List<MesBom> mesBoms = getBomList(mesPart.getOrganizeCode(), mesPart, mesProductVersion.getAlternativePartList());
        //保存数据
        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
        for (MesBom mesBom : mesBoms) {
            mesProductOffLineList.add(getMesProductOffLine(model, mesPart, mesProductVersion, mesBom));
        }
        mesProductOffLineService.insertBatch(mesProductOffLineList);
    }

    private MesProductOffLine getMesProductOffLine(WmsProductModel model, MesPart mesPart, MesProductVersion mesProductVersion, MesBom mesBom) {
        MesProductOffLine newMesProductOffLine = new MesProductOffLine();
        newMesProductOffLine.setReportPartNo(mesPart.getPartNo());
        newMesProductOffLine.setReportPartNameRdd(mesPart.getPartName());
        newMesProductOffLine.setReportSn(model.getSn());
        newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
        newMesProductOffLine.setItemPartName(mesBom.getItemPartName());
        newMesProductOffLine.setItemQty(MathOperation.mul(model.getQty(), mesBom.getItemQty()));
        newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
        newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
        newMesProductOffLine.setQty(model.getQty());
        newMesProductOffLine.setBomVersion(mesProductVersion.getProductVersion());
        newMesProductOffLine.setSerialNumber(model.getSn());
        newMesProductOffLine.setUnit(mesBom.getUnit());
        newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
        newMesProductOffLine.setReportType(MesExtEnumUtil.REPORT_TYPE.CBR.getValue());
        newMesProductOffLine.setOrganizeCode(model.getOrganizeCode());
        newMesProductOffLine.setOrderNo(model.getShipOrderNo());
        newMesProductOffLine.setCustNo(model.getCustNo());
        newMesProductOffLine.setCustOrder(model.getCustOrderNo());

        ConvertBean.serviceModelInitialize(newMesProductOffLine, model.getUserName());
        return newMesProductOffLine;
    }

    private MesProductVersion getMesProductVersion(WmsProductModel model) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(model.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(model.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getOrderBy(null, null, ddlPackBean);
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (null == mesProductVersion) {
            MesException.throwMesBusiException("物料【%s】生产版本信息不存在", model.getPartNo());
        }
        return mesProductVersion;
    }

    private void customerSupplyMove(MesWorkOrder mesWorkOrder, MesProductVersion mesProductVersion, List<MesBom> mesBoms, List<String> resultList, List<MesProductOffLine> mesProductOffLineList, List<MesMove> mesMoveList) {
        boolean isItemMove;
        boolean isItemReport;
        MesMoveRule moveRule;
        //查询零件生产组的移库规则
        List<MesMoveRule> moveRules = mesMoveRuleService.findMesMoveRuleByPartProdGroupCode(mesWorkOrder.getPartProdGroupCode(), mesWorkOrder.getOrganizeCode());
        Map<String, List<MesMoveRule>> mesMoveRuleMap = moveRules.stream().filter(t -> Objects.nonNull(t.getSrcType())).collect(Collectors.groupingBy(MesMoveRule::getSrcType));
        List<String> itemPartNoList = mesBoms.stream().map(MesBom::getItemPartNo).collect(Collectors.toList());
        //子物料SAP下发信息
        Map<String, List<MesPartSap>> mesPartSapMap = getPartSapMap(mesWorkOrder, itemPartNoList);
        //获取物料信息
        Map<String, List<MesPart>> itemPartMap = getItemPartMap(mesWorkOrder, itemPartNoList);
        for (String sn : resultList) {
            for (MesBom mesBom : mesBoms) {
                isItemReport = false;
                isItemMove = false;
                moveRule = null;
                if (itemPartMap.containsKey(mesBom.getItemPartNo())) {
                    MesPart itemPart = itemPartMap.get(mesBom.getItemPartNo()).iterator().next();
                    if (!StringUtil.isEmpty(itemPart.getEsd()) && mesMoveRuleMap.containsKey(itemPart.getEsd())) {
                        moveRule = mesMoveRuleMap.get(itemPart.getEsd()).iterator().next();
                        if (MesExtEnumUtil.MOVE_TYPE_REPORT_TYPE.REPORT_MOVE.getValue() == moveRule.getReportType()) {
                            isItemMove = true;
                            isItemReport = true;
                        } else if (MesExtEnumUtil.MOVE_TYPE_REPORT_TYPE.REPORT.getValue() == moveRule.getReportType()) {
                            isItemReport = true;
                        } else if (MesExtEnumUtil.MOVE_TYPE_REPORT_TYPE.MOVE.getValue() == moveRule.getReportType()) {
                            isItemMove = true;
                        }
                    }
                }
                //汇报
                if (isItemReport) {
                    mesProductOffLineList.add(creatMesProductOffLine(mesWorkOrder, mesProductVersion, sn, mesBom, true));
                }
                //移库
                if (isItemMove) {
                    MesPartSap mesPartSap = null;
                    if (mesPartSapMap.containsKey(mesBom.getItemPartNo())) {
                        mesPartSap = mesPartSapMap.get(mesBom.getItemPartNo()).iterator().next();
                    }
                    MesMove move = createMove(mesPartSap, moveRule.getErpSrcLocateNo(), moveRule.getErpDestLocateNo(), mesBom.getOrganizeCode(), mesBom.getItemQty(), "", MesExtEnumUtil.MOVE_TYPE.RAW_MATERIAL_MOVE.getValue());
                    move.setMatnr(mesBom.getItemPartNo());
                    mesMoveList.add(move);
                }
            }
        }
    }

    private Map<String, List<MesPartSap>> getPartSapMap(MesWorkOrder mesWorkOrder, List<String> itemPartNoList) {
        List<MesPartSap> mesPartSapList = mesPartSapService.findMesPartSapByPartNoList(itemPartNoList, mesWorkOrder.getOrganizeCode());
        Map<String, List<MesPartSap>> mesPartSapMap = mesPartSapList.stream().filter(t -> Objects.nonNull(t.getPartNo())).collect(Collectors.groupingBy(MesPartSap::getPartNo));
        return mesPartSapMap;
    }

    private Map<String, List<MesPart>> getItemPartMap(MesWorkOrder mesWorkOrder, List<String> itemPartNoList) {
        List<MesPart> itemPartList = iMesPartService.findMesPartByPartNoList(itemPartNoList, mesWorkOrder.getOrganizeCode());
        Map<String, List<MesPart>> itemPartMap = itemPartList.stream().filter(t -> Objects.nonNull(t.getPartNo())).collect(Collectors.groupingBy(MesPart::getPartNo));
        return itemPartMap;
    }

    private MesProdRuleSortCfg getMesProdRuleSortCfg(MesWorkOrder mesWorkOrder) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getPartNo(), "reportPartNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        MesProdRuleSortCfg mesProdRuleSortCfg = mesProdRuleSortCfgRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesProdRuleSortCfg)) {
            MesException.throwMesBusiException("产线【%s】物料【%s】排序加工规则未维护", mesWorkOrder.getWorkCenterCode(), mesWorkOrder.getPartNo());
        }
        return mesProdRuleSortCfg;
    }

    private List<String> getStringList(MesWorkOrder mesWorkOrder) {
        GenSerialNoModel genSerialNoModel = new GenSerialNoModel(MesCommonConstant.REPORT_SN);
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, mesWorkOrder.getCreateUser(), mesWorkOrder.getOrganizeCode(), mesWorkOrder.getNum()).getResultList();
        return resultList;
    }

    private List<MesBom> getMesBoms(String partNo, String organizeCode, String bomVersion) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(bomVersion, "bomVersion", ddlPackBean);
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime(true), "effStartTime", ddlPackBean);
        DdlPreparedPack.getOrderBy("effStartTime", CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), ddlPackBean);
        List<MesBom> mesBoms = mesBomRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesBoms)) {
            MesException.throwMesBusiException("物料【%s】生产版本【%s】对应bom信息不存在", partNo, bomVersion);
        }
        MesBom next = mesBoms.iterator().next();
        //如果存在多个bomCode取最新的一个
        return mesBoms.stream().collect(Collectors.groupingBy(MesBom::getBomCode)).get(next.getBomCode());
    }

    private MesProductVersion getMesProductVersion(MesWorkOrder mesWorkOrder) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getErpWorkCenter(), "workCenterCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getProductVersion(), "productVersion", ddlPackBean);
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (null == mesProductVersion) {
            MesException.throwMesBusiException("物料【%s】生产版本信息不存在", mesWorkOrder.getPartNo());
        }
        return mesProductVersion;
    }


    private MesProductOffLine creatMesProductOffLine(MesWorkOrder mesWorkOrder, MesProductVersion mesProductVersion,
                                                     String sn, MesBom mesBom, boolean isItemReport) {
        MesProductOffLine newMesProductOffLine;
        newMesProductOffLine = new MesProductOffLine();
        if (!isItemReport) {
            newMesProductOffLine.setReportPartNo(mesWorkOrder.getPartNo());
            newMesProductOffLine.setReportPartNameRdd(mesWorkOrder.getPartName());
        }

        newMesProductOffLine.setReportSn(sn);
        newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
        newMesProductOffLine.setItemPartName(mesBom.getItemPartName());
        newMesProductOffLine.setItemQty(mesBom.getItemQty());
        newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
        newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
        newMesProductOffLine.setQty(1d);
        newMesProductOffLine.setBomVersion(mesWorkOrder.getProductVersion());
        newMesProductOffLine.setSerialNumber(sn);
        newMesProductOffLine.setUnit(mesBom.getUnit());
        newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
        newMesProductOffLine.setWorkOrderNo(mesWorkOrder.getWorkOrderNo());
        newMesProductOffLine.setWorkCenterCode(mesWorkOrder.getWorkCenterCode());
        newMesProductOffLine.setWorkCellCode(mesWorkOrder.getWorkCellCode());
        newMesProductOffLine.setReportType(mesWorkOrder.getReportType());
        newMesProductOffLine.setSapWorkCenter(mesWorkOrder.getErpWorkCenter());
        newMesProductOffLine.setOrganizeCode(mesWorkOrder.getOrganizeCode());

        ConvertBean.serviceModelInitialize(newMesProductOffLine, mesWorkOrder.getCreateUser());
        return newMesProductOffLine;
    }

    @Override
    public List<MesWorkOrder> sumMesWorkOrderQtyToSap(String organizeCode) {
        List<Integer> statusList = new ArrayList<>();
        statusList.add(MesExtEnumUtil.ORDER_STATUS.CREATE.getValue());
//        statusList.add(MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue());
        //查询汇总工单数量
        return mesWorkOrderDao.sumMesWorkOrderQtyToSap(organizeCode, statusList);
    }

    @Override
    public List<MesWorkOrder> queryMesWorkOrderProcessToSap(String organizeCode) {
        List<Integer> statusList = new ArrayList<>();
        statusList.add(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
//        statusList.add(MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue());
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);

        DdlPreparedPack.getInPackList(statusList, "workOrderStatus", packBean);
        return mesWorkOrderRDao.findByHqlWhere(packBean);
    }

    private MesMove createMove(MesPartSap mesPart, String source, String target, String org, double qty, String sn, Integer moveType) {
        MesMove move = new MesMove();
        if (!Objects.isNull(mesPart)) {
            move.setMeins(mesPart.getUnit());
        }
        move.setOrganizeCode(org);
        move.setFactoryCode(org);
        move.setLgort(source);
        move.setUmlgo(target);
        move.setMenge(qty);
        move.setPostDate(TimeTool.getToday());
        move.setProductSn(sn);
        move.setMoveType(moveType);
        move.setPostTime(TimeTool.getTimeShortWithColon());
        ConvertBean.serviceModelInitialize(move, AuthUtil.getSessionUser().getUserName());
        return move;
    }

    /**
     * 提供（相对）精确的除法运算。 当发生除不尽的情况时，由scale参数指定精度，以后的数字四舍五入。
     *
     * @param dividend 被除数
     * @param divisor  除数
     * @param scale    表示表示需要精确到小数点以后几位。
     * @return 两个参数的商
     */
    public Double divide(Double dividend, Double divisor, Integer scale) {
        if (scale < 0) {
            throw new IllegalArgumentException("The scale must be a positive integer or zero");
        }
        BigDecimal b1 = new BigDecimal(Double.toString(dividend));
        BigDecimal b2 = new BigDecimal(Double.toString(divisor));
        return b1.divide(b2, scale, RoundingMode.HALF_UP).doubleValue();
    }
}
