package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.mes.api.iservice.busi.IExcelImportService;
import cn.estsh.i3plus.mes.apiservice.serviceimpl.busi.CommonService;
import cn.estsh.i3plus.mes.apiservice.util.MesCommonUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerCarModel;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerCarModelDetail;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerOriganze;
import cn.estsh.i3plus.pojo.mes.model.ExcelImportErrorModel;
import cn.estsh.i3plus.pojo.mes.model.ExcelImportResultModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 车型配置
 */
@Service
@Slf4j
public class MesCustomerCarModelExcelService implements IExcelImportService {

    @Autowired
    private CommonService commonService;

    @Autowired
    private MesCustomerRepository customerRepository;
    @Autowired
    private MesCustomerOriganizeRepository customerOriganizeRepository;

    @Autowired
    private MesCustomerPartRepository customerPartRepository;
    @Autowired
    private MesPartRepository partRepository;
    @Autowired
    private MesCustomerCarModelRepository customerCarModelRepository;

    @Override
    public ExcelImportResultModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {

        //读取表格
        ExcelImportResultModel excelImportResultModel = this.sheetExtractMesStationBom(workbook.getSheetAt(0), organizeCode, userName);
//        //数据入库
//        this.insertExcelMesItemPartErrorPrevention(excelImportResultModel);
        return excelImportResultModel;
//        return null;
    }

    /**
     * 工位BOM表导入-读取表格
     *
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultModel sheetExtractMesStationBom(Sheet sheetAt, String organizeCode, String userName) {

        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //客户车型信息集合
        List<MesCustomerCarModel> modelList = new ArrayList<>();
        List<MesCustomerCarModelDetail> modelDetailList = new ArrayList<>();

        // 查询客户信息表
        Map<String, Object> customerMap;
        try {
            customerMap = MesCommonUtil.getCodeEntityMap(customerRepository, "custCode", organizeCode, "客户信息表");
        } catch (Exception e) {
            customerMap = null;
        }
        // 查询客户工厂信息
        Map<String, Object> customerOriganizeMap;
        try {
            customerOriganizeMap = MesCommonUtil.getCodeEntityMap(customerOriganizeRepository, "custOrganizeCode", organizeCode, "客户工厂信息表");
        } catch (Exception e) {
            customerOriganizeMap = null;
        }
        // 查询客户零件关系
        Map<String, Object> customerPartMap;
        try {
            customerPartMap = MesCommonUtil.getCodeEntityMap(customerPartRepository, "custPartNo", organizeCode, "客户零件关系表");
        } catch (Exception e) {
            customerPartMap = null;
        }
        // 查询客户零件关系
        Map<String, Object> partMap;
        try {
            partMap = MesCommonUtil.getCodeEntityMap(partRepository, "partNo", organizeCode, "物料信息表");
        } catch (Exception e) {
            partMap = null;
        }

//        //查询关键件条码校验规则表
//        List<MesItemPartErrorPrevention> itemPartErrorPreventionList = itemPartErrorPreventionRepository.findByProperty(new String[]{MesConstWords.ORGANIZE_CODE, MesConstWords.IS_DELETED},
//                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
//        Map<String, List<MesItemPartErrorPrevention>> preventionMap =
//                itemPartErrorPreventionList.stream().collect(Collectors.groupingBy(k -> k.getPreventionObjectCode() + "&" + k.getPartNo()));

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorModel> excelImportErrorModels = new ArrayList<>();


        Map<String, String> map = new HashMap<>();

        for (int i = (sheetAt.getFirstRowNum() + 1); i < totalNumberOfRows; i++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //客户代码
                String custCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();

                //客户工厂代码
                String custOrganizeCode = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    custOrganizeCode = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }


                //车型代码
                String carModelCode = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    carModelCode = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //车型名称
                String carModelName = null;
                try {
                    row.getCell(3).setCellType(CellType.STRING);
                    carModelName = row.getCell(3, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //客户零件号
                String customerPartNo = null;
                try {
                    row.getCell(4).setCellType(CellType.STRING);
                    customerPartNo = row.getCell(4, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //备注
                String remark = null;
                try {
                    row.getCell(5).setCellType(CellType.STRING);
                    remark = row.getCell(5, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //是否罕见车型
                String isUncommon = null;
                try {
                    row.getCell(6).setCellType(CellType.STRING);
                    isUncommon = row.getCell(6, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //零件号
                String partNo = null;
                try {
                    row.getCell(7).setCellType(CellType.STRING);
                    partNo = row.getCell(7, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //供货零件号
                String supperPartNo = null;
                try {
                    row.getCell(8).setCellType(CellType.STRING);
                    supperPartNo = row.getCell(8, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //供货数量
                String qty = null;
                try {
                    row.getCell(9).setCellType(CellType.STRING);
                    qty = row.getCell(9, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //校验客户代码
                if (StringUtils.isEmpty(custCode)) {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据必填;";
                } else {
                    if (CollectionUtils.isEmpty(customerMap) ||
                            (!CollectionUtils.isEmpty(customerMap) && !customerMap.containsKey(custCode))) {
                        errorNum++;
                        cellNum += "A;";
                        errorInfo += "第A列数据无效;";
                    }
                }
                //校验客户代码
                if (StringUtils.isEmpty(custOrganizeCode)) {
                    errorNum++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                } else {
                    if (CollectionUtils.isEmpty(customerOriganizeMap) ||
                            (!CollectionUtils.isEmpty(customerOriganizeMap) && !customerOriganizeMap.containsKey(custOrganizeCode))) {
                        errorNum++;
                        cellNum += "B;";
                        errorInfo += "第B列数据无效;";
                    }
                }

                //校验车型代码
                if (StringUtils.isEmpty(carModelCode)) {
                    errorNum++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                }

                //校验车型名称
                if (StringUtils.isEmpty(carModelName)) {
                    errorNum++;
                    cellNum += "D;";
                    errorInfo += "第D列数据必填;";
                }

                //校验客户零件号
                if (StringUtils.isEmpty(customerPartNo)) {
                    errorNum++;
                    cellNum += "E;";
                    errorInfo += "第E列数据必填:";
                } else {
                    if (CollectionUtils.isEmpty(customerPartMap) ||
                            (!CollectionUtils.isEmpty(customerPartMap) && !customerPartMap.containsKey(customerPartNo))) {
                        errorNum++;
                        cellNum += "E;";
                        errorInfo += "第E列数据无效;";
                    }
                }
                //校验客户零件号
                if (StringUtils.isEmpty(partNo)) {
                    errorNum++;
                    cellNum += "G;";
                    errorInfo += "第G列数据必填:";
                } else {
                    if (CollectionUtils.isEmpty(partMap) ||
                            (!CollectionUtils.isEmpty(partMap) && !partMap.containsKey(partNo))) {
                        errorNum++;
                        cellNum += "G;";
                        errorInfo += "第G列数据无效;";
                    }
                }
                //校验供货零件号
                if (StringUtils.isEmpty(supperPartNo)) {
                    errorNum++;
                    cellNum += "H;";
                    errorInfo += "第H列数据必填:";
                } else {
                    if (CollectionUtils.isEmpty(partMap) ||
                            (!CollectionUtils.isEmpty(partMap) && !partMap.containsKey(supperPartNo))) {
                        errorNum++;
                        cellNum += "H;";
                        errorInfo += "第H列数据无效;";
                    }
                }
                //校验客户零件号
                if (StringUtils.isEmpty(qty)) {
                    errorNum++;
                    cellNum += "H;";
                    errorInfo += "第H列数据必填:";
                } else {
                    //是否为数字
                    if (!NumberUtils.isDigits(qty)) {
                        errorNum++;
                        cellNum += "H;";
                        errorInfo += "第H列数据必须为数字;";
                    }
                }

                //没有错误的时候，封装MesStationBom
                if (errorNum == 0) {

                    MesCustomerCarModel model = new MesCustomerCarModel();
                    model.setCarModelCode(carModelCode);
                    model.setCarModelName(carModelName);
                    model.setCustCode(custCode);
                    model.setCustOrganizeCode(custOrganizeCode);
                    MesCustomerOriganze customerOriganze = (MesCustomerOriganze) customerOriganizeMap.get(custOrganizeCode);
                    model.setCustOrganizeName(customerOriganze.getCustOrganizeName());
                    model.setRemark(remark);
                    model.setIsUncommon(Integer.parseInt(isUncommon));
                    model.setOrganizeCode(organizeCode);
                    ConvertBean.serviceModelInitialize(model, AuthUtil.getSessionUser().getUserName());
                    modelList.add(model);

                    MesCustomerCarModelDetail modelDetail = new MesCustomerCarModelDetail();
                    modelDetail.setOrganizeCode(organizeCode);
                    modelDetail.setCustPartNo(customerPartNo);
                    modelDetail.setPartNo(partNo);
                    modelDetail.setSupperPartNo(supperPartNo);
//                    modelDetail.setQty(Integer.parseInt(qty));
                    ConvertBean.serviceModelInitialize(modelDetail, AuthUtil.getSessionUser().getUserName());
                    modelDetailList.add(modelDetail);

                    successRowNum++;
                } else {
                    //封装错误行信息ExcelImportErrorModel
                    excelImportErrorModels = commonService.getExcelImportErrorModels(excelImportErrorModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum++;
                }

            }
        }

        //校验EXCEL数据
        commonService.checkExcelData(failRowNum, successRowNum, errorRows);
        //封装返回结果
        ExcelImportResultModel excelImportResultModel = commonService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : modelList);
        if (CollectionUtils.isEmpty(modelList)) {
            return excelImportResultModel;
        } else {
            //数据入库
            modelList.forEach(k -> {
                if (StringUtils.isEmpty(k.getId())) {
//                itemPartErrorPreventionRepository.insert(k);
                } else {
//                itemPartErrorPreventionRepository.update(k);
                }
            });
        }
        return excelImportResultModel;
    }

}
