package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.mes.apiservice.util.DateUtil;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.util.BaseRedis;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFDataFormat;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;

import javax.persistence.EntityManager;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @Description:
 * @CreateDate 2022/6/27 17:28
 * @Author 尹肖
 */
public class MesExcelTool extends ExcelTool {
    public MesExcelTool(EntityManager entityManager, BaseRedis redisRes) {
        super(entityManager, redisRes);
    }

    @Override
    protected Object getCellValue(Cell cell, Class cellClass) {
        Object value = "";
        if (cell == null) {
            return null;
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC) && HSSFDateUtil.isCellDateFormatted(cell)) {
            value = this.getDateCellValue(cell, cellClass);
            return value;
        } else {
            cell.setCellType(CellType.STRING);
            if ("null".equals(cell.getStringCellValue())) {
                value = "";
            } else if ((cellClass == Integer.class || cellClass == Long.class || cellClass == Double.class || cellClass == Float.class) && StringUtils.isBlank(cell.getStringCellValue())) {
                value = null;
            } else if (cellClass == String.class) {
                value = cell.getStringCellValue();
            } else if (cellClass == Integer.class) {
                value = Integer.parseInt(cell.getStringCellValue());
            } else if (cellClass == Long.class) {
                value = Long.parseLong(cell.getStringCellValue());
            } else if (cellClass == Double.class) {
                value = Double.parseDouble(cell.getStringCellValue());
            } else if (cellClass == Float.class) {
                value = Float.parseFloat(cell.getStringCellValue());
            }

            return value;
        }
    }

    Object getDateCellValue(Cell cell, Class cellClass) {
        Object value = null;
        if (cell.getCellTypeEnum().equals(CellType.NUMERIC) && HSSFDateUtil.isCellDateFormatted(cell)) {
            if (cell.getCellStyle().getDataFormat() == HSSFDataFormat.getBuiltinFormat("h:mm:ss")) {
                value = TimeTool.pareDateToString("HH:mm:ss", cell.getDateCellValue());
            } else if (cell.getCellStyle().getDataFormat() == HSSFDataFormat.getBuiltinFormat("yyyy-MM-dd")) {
                value = TimeTool.pareDateToString("yyyy-MM-dd", cell.getDateCellValue());
            } else if (cell.getCellStyle().getDataFormat() == 58) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                double numericCellValue = cell.getNumericCellValue();
                Date date = HSSFDateUtil.getJavaDate(numericCellValue);
                value = sdf.format(date);
            } else {
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat(DateUtil.BASE_FORMAT);
                double numericCellValue = cell.getNumericCellValue();
                value = simpleDateFormat.format(HSSFDateUtil.getJavaDate(numericCellValue));
            }
        }

        return value;
    }
}
